searchNodes=[{"doc":"The plug specification.There are two kind of plugs: function plugs and module plugs.Function plugsA function plug is any function that receives a connection and a set of options and returns a connection. Its type signature must be:(Plug.Conn.t, Plug.opts) :: Plug.Conn.tModule plugsA module plug is an extension of the function plug. It is a module that must export:a call/2 function with the signature defined abovean init/1 function which takes a set of options and initializes it.The result returned by init/1 is passed as second argument to call/2. Note that init/1 may be called during compilation and as such it must not return pids, ports or values that are specific to the runtime.The API expected by a module plug is defined as a behaviour by the Plug module (this module).ExamplesHere's an example of a function plug:def json_header_plug(conn, opts) do Plug.Conn.put_resp_content_type(conn, &quot;application/json&quot;) endHere's an example of a module plug:defmodule JSONHeaderPlug do import Plug.Conn def init(opts) do opts end def call(conn, _opts) do put_resp_content_type(conn, &quot;application/json&quot;) end endThe Plug pipelineThe Plug.Builder module provides conveniences for building plug pipelines.","ref":"Plug.html","title":"Plug","type":"behaviour"},{"doc":"","ref":"Plug.html#c:call/2","title":"Plug.call/2","type":"callback"},{"doc":"Forwards requests to another Plug setting the connection to a trailing subpath of the request.The path_info on the forwarded connection will only include the trailing segments of the request path supplied to forward, while conn.script_name will retain the correct base path for e.g. url generation.Exampledefmodule Router do def init(opts), do: opts def call(conn, opts) do case conn do # Match subdomain %{host: &quot;admin.&quot; &lt;&gt; _} -&gt; AdminRouter.call(conn, opts) # Match path on localhost %{host: &quot;localhost&quot;, path_info: [&quot;admin&quot; | rest]} -&gt; Plug.forward(conn, rest, AdminRouter, opts) _ -&gt; MainRouter.call(conn, opts) end end end","ref":"Plug.html#forward/4","title":"Plug.forward/4","type":"function"},{"doc":"","ref":"Plug.html#c:init/1","title":"Plug.init/1","type":"callback"},{"doc":"Run a series of Plugs at runtime.The plugs given here can be either a tuple, representing a module plug and their options, or a simple function that receives a connection and returns a connection.If any of the plugs halt, the remaining plugs are not invoked. If the given connection was already halted, none of the plugs are invoked either.While Plug.Builder works at compile-time, this is a straight-forward alternative that works at runtime.ExamplesPlug.run(conn, [{Plug.Head, []}, &amp;IO.inspect/1])Options:log_on_halt - a log level to be used if a Plug halts","ref":"Plug.html#run/3","title":"Plug.run/3","type":"function"},{"doc":"","ref":"Plug.html#t:opts/0","title":"Plug.opts/0","type":"type"},{"doc":"Conveniences for building plugs.This module can be use-d into a module in order to build a plug pipeline:defmodule MyApp do use Plug.Builder plug Plug.Logger plug :hello, upper: true # A function from another module can be plugged too, provided it&#39;s # imported into the current module first. import AnotherModule, only: [interesting_plug: 2] plug :interesting_plug def hello(conn, opts) do body = if opts[:upper], do: &quot;WORLD&quot;, else: &quot;world&quot; send_resp(conn, 200, body) end endMultiple plugs can be defined with the plug/2 macro, forming a pipeline. The plugs in the pipeline will be executed in the order they've been added through the plug/2 macro. In the example above, Plug.Logger will be called first and then the :hello function plug will be called on the resulting connection.Plug.Builder also imports the Plug.Conn module, making functions like send_resp/3 available.OptionsWhen used, the following options are accepted by Plug.Builder::log_on_halt - accepts the level to log whenever the request is halted:init_mode - the environment to initialize the plug's options, one of :compile or :runtime. Defaults :compile.Plug behaviourInternally, Plug.Builder implements the Plug behaviour, which means both the init/1 and call/2 functions are defined.By implementing the Plug API, Plug.Builder guarantees this module is a plug and can be handed to a web server or used as part of another pipeline.Overriding the default Plug API functionsBoth the init/1 and call/2 functions defined by Plug.Builder can be manually overridden. For example, the init/1 function provided by Plug.Builder returns the options that it receives as an argument, but its behaviour can be customized:defmodule PlugWithCustomOptions do use Plug.Builder plug Plug.Logger def init(opts) do opts end endThe call/2 function that Plug.Builder provides is used internally to execute all the plugs listed using the plug macro, so overriding the call/2 function generally implies using super in order to still call the plug chain:defmodule PlugWithCustomCall do use Plug.Builder plug Plug.Logger plug Plug.Head def call(conn, opts) do conn |&gt; super(opts) # calls Plug.Logger and Plug.Head |&gt; assign(:called_all_plugs, true) end endHalting a plug pipelineA plug pipeline can be halted with Plug.Conn.halt/1. The builder will prevent further plugs downstream from being invoked and return the current connection. In the following example, the Plug.Logger plug never gets called:defmodule PlugUsingHalt do use Plug.Builder plug :stopper plug Plug.Logger def stopper(conn, _opts) do halt(conn) end end","ref":"Plug.Builder.html","title":"Plug.Builder","type":"module"},{"doc":"Annotates a plug will receive the options given to the current module itself as arguments.Imagine the following plug:defmodule MyPlug do use Plug.Builder plug :inspect_opts, builder_opts() defp inspect_opts(conn, opts) do IO.inspect(opts) conn end endWhen plugged as:plug MyPlug, custom: :optionsIt will print [custom: :options] as the builder options were passed to the inner plug.Note you only pass builder_opts() to function plugs. You cannot use builder_opts() with module plugs because their options are evaluated at compile time. If you need to pass builder_opts() to a module plug, you can wrap the module plug in function. To be precise, do not do this:plug Plug.Parsers, builder_opts()Instead do this:plug :custom_plug_parsers, builder_opts() defp custom_plug_parsers(conn, opts) do Plug.Parsers.call(conn, Plug.Parsers.init(opts)) end","ref":"Plug.Builder.html#builder_opts/0","title":"Plug.Builder.builder_opts/0","type":"macro"},{"doc":"Compiles a plug pipeline.Each element of the plug pipeline (according to the type signature of this function) has the form:{plug_name, options, guards}Note that this function expects a reversed pipeline (with the last plug that has to be called coming first in the pipeline).The function returns a tuple with the first element being a quoted reference to the connection and the second element being the compiled quoted pipeline.ExamplesPlug.Builder.compile(env, [ {Plug.Logger, [], true}, # no guards, as added by the Plug.Builder.plug/2 macro {Plug.Head, [], quote(do: a when is_binary(a))} ], [])","ref":"Plug.Builder.html#compile/3","title":"Plug.Builder.compile/3","type":"function"},{"doc":"A macro that stores a new plug. opts will be passed unchanged to the new plug.This macro doesn't add any guards when adding the new plug to the pipeline; for more information about adding plugs with guards see compile/3.Examplesplug Plug.Logger # plug module plug :foo, some_options: true # plug function","ref":"Plug.Builder.html#plug/2","title":"Plug.Builder.plug/2","type":"macro"},{"doc":"","ref":"Plug.Builder.html#t:plug/0","title":"Plug.Builder.plug/0","type":"type"},{"doc":"The Plug connection.This module defines a struct and the main functions for working with requests and responses in an HTTP connection.Note request headers are normalized to lowercase and response headers are expected to have lowercase keys.Request fieldsThese fields contain request information:host - the requested host as a binary, example: &quot;www.example.com&quot;method - the request method as a binary, example: &quot;GET&quot;path_info - the path split into segments, example: [&quot;hello&quot;, &quot;world&quot;]script_name - the initial portion of the URL's path that corresponds to the application routing, as segments, example: [&quot;sub&quot;,&quot;app&quot;]request_path - the requested path, example: /trailing/and//double//slashes/port - the requested port as an integer, example: 80remote_ip - the IP of the client, example: {151, 236, 219, 228}. This field is meant to be overwritten by plugs that understand e.g. the X-Forwarded-For header or HAProxy's PROXY protocol. It defaults to peer's IPreq_headers - the request headers as a list, example: [{&quot;content-type&quot;, &quot;text/plain&quot;}]. Note all headers will be downcasedscheme - the request scheme as an atom, example: :httpquery_string - the request query string as a binary, example: &quot;foo=bar&quot;Fetchable fieldsThe request information in these fields is not populated until it is fetched using the associated fetch_ function. For example, the cookies field uses fetch_cookies/2.If you access these fields before fetching them, they will be returned as Plug.Conn.Unfetched structs.cookies- the request cookies with the response cookiesbody_params - the request body params, populated through a Plug.Parsers parser.query_params - the request query params, populated through fetch_query_params/2path_params - the request path params, populated by routers such as Plug.Routerparams - the request params, the result of merging the :path_params on top of :body_params on top of :query_paramsreq_cookies - the request cookies (without the response ones)Response fieldsThese fields contain response information:resp_body - the response body, by default is an empty string. It is set to nil after the response is sent, except for test connections. The response charset used defaults to &quot;utf-8&quot;.resp_cookies - the response cookies with their name and optionsresp_headers - the response headers as a list of tuples, by default cache-control is set to &quot;max-age=0, private, must-revalidate&quot;. Note, response headers are expected to have lowercase keys.status - the response statusFurthermore, the before_send field stores callbacks that are invoked before the connection is sent. Callbacks are invoked in the reverse order they are registered (callbacks registered first are invoked last) in order to reproduce a pipeline ordering.Connection fieldsassigns - shared user data as a mapowner - the Elixir process that owns the connectionhalted - the boolean status on whether the pipeline was haltedsecret_key_base - a secret key used to verify and encrypt cookies. the field must be set manually whenever one of those features are used. This data must be kept in the connection and never used directly, always use Plug.Crypto.KeyGenerator.generate/3 to derive keys from itstate - the connection stateThe connection state is used to track the connection lifecycle. It starts as :unset but is changed to :set (via resp/3) or :set_chunked (used only for before_send callbacks by send_chunked/2) or :file (when invoked via send_file/3). Its final result is :sent, :file or :chunked depending on the response model.Private fieldsThese fields are reserved for libraries/framework usage.adapter - holds the adapter information in a tupleprivate - shared library data as a mapCustom status codesPlug allows status codes to be overridden or added in order to allow new codes not directly specified by Plug or its adapters. Adding or overriding a status code is done through the Mix configuration of the :plug application. For example, to override the existing 404 reason phrase for the 404 status code (&quot;Not Found&quot; by default) and add a new 998 status code, the following config can be specified:config :plug, :statuses, %{ 404 =&gt; &quot;Actually This Was Found&quot;, 998 =&gt; &quot;Not An RFC Status Code&quot; }As this configuration is Plug specific, Plug will need to be recompiled for the changes to take place: this will not happen automatically as dependencies are not automatically recompiled when their configuration changes. To recompile Plug:mix deps.clean --build plugThe atoms that can be used in place of the status code in many functions are inflected from the reason phrase of the status code. With the above configuration, the following will all work:put_status(conn, :not_found) # 404 put_status(conn, :actually_this_was_found) # 404 put_status(conn, :not_an_rfc_status_code) # 998Even though 404 has been overridden, the :not_found atom can still be used to set the status to 404 as well as the new atom :actually_this_was_found inflected from the reason phrase &quot;Actually This Was Found&quot;.","ref":"Plug.Conn.html","title":"Plug.Conn","type":"module"},{"doc":"Assigns a value to a key in the connection.The &quot;assigns&quot; storage is meant to be used to store values in the connection so that other plugs in your plug pipeline can access them. The assigns storage is a map.Examplesiex&gt; conn.assigns[:hello] nil iex&gt; conn = assign(conn, :hello, :world) iex&gt; conn.assigns[:hello] :world","ref":"Plug.Conn.html#assign/3","title":"Plug.Conn.assign/3","type":"function"},{"doc":"Sends a chunk as part of a chunked response.It expects a connection with state :chunked as set by send_chunked/2. It returns {:ok, conn} in case of success, otherwise {:error, reason}.To stream data use Enum.reduce_while/3 instead of Enum.into/2. Enum.reduce_while/3 allows aborting the execution if chunk/2 fails to deliver the chunk of data.ExampleEnum.reduce_while(~w(each chunk as a word), conn, fn (chunk, conn) -&gt; case Plug.Conn.chunk(conn, chunk) do {:ok, conn} -&gt; {:cont, conn} {:error, :closed} -&gt; {:halt, conn} end end)","ref":"Plug.Conn.html#chunk/2","title":"Plug.Conn.chunk/2","type":"function"},{"doc":"Clears the entire session.This function removes every key from the session, clearing the session.Note that, even if clear_session/1 is used, the session is still sent to the client. If the session should be effectively dropped, configure_session/2 should be used with the :drop option set to true.","ref":"Plug.Conn.html#clear_session/1","title":"Plug.Conn.clear_session/1","type":"function"},{"doc":"Configures the session.Options:renew - generates a new session id for the cookie:drop - drops the session, a session cookie will not be included in the response:ignore - ignores all changes made to the session in this request cycle","ref":"Plug.Conn.html#configure_session/2","title":"Plug.Conn.configure_session/2","type":"function"},{"doc":"Deletes a request header if present.Raises a Plug.Conn.AlreadySentError if the connection has already been :sent or :chunked.ExamplesPlug.Conn.delete_req_header(conn, &quot;content-type&quot;)","ref":"Plug.Conn.html#delete_req_header/2","title":"Plug.Conn.delete_req_header/2","type":"function"},{"doc":"Deletes a response cookie.Deleting a cookie requires the same options as to when the cookie was put. Check put_resp_cookie/4 for more information.","ref":"Plug.Conn.html#delete_resp_cookie/3","title":"Plug.Conn.delete_resp_cookie/3","type":"function"},{"doc":"Deletes a response header if present.Raises a Plug.Conn.AlreadySentError if the connection has already been :sent or :chunked.ExamplesPlug.Conn.delete_resp_header(conn, &quot;content-type&quot;)","ref":"Plug.Conn.html#delete_resp_header/2","title":"Plug.Conn.delete_resp_header/2","type":"function"},{"doc":"Deletes key from session.The key can be a string or an atom, where atoms are automatically converted to strings.","ref":"Plug.Conn.html#delete_session/2","title":"Plug.Conn.delete_session/2","type":"function"},{"doc":"Fetches cookies from the request headers.Options:signed - a list of one or more cookies that are signed and must be verified accordingly:encrypted - a list of one or more cookies that are encrypted and must be decrypted accordinglySee put_resp_cookie/4 for more information.","ref":"Plug.Conn.html#fetch_cookies/2","title":"Plug.Conn.fetch_cookies/2","type":"function"},{"doc":"Fetches query parameters from the query string.Params are decoded as &quot;x-www-form-urlencoded&quot; in which key/value pairs are separated by &amp; and keys are separated from values by =.This function does not fetch parameters from the body. To fetch parameters from the body, use the Plug.Parsers plug.Options:length - the maximum query string length. Defaults to 1_000_000 bytes.:validate_utf8 - boolean that tells whether or not to validate the keys and values of the decoded query string are UTF-8 encoded. Defaults to true.","ref":"Plug.Conn.html#fetch_query_params/2","title":"Plug.Conn.fetch_query_params/2","type":"function"},{"doc":"Fetches the session from the session store. Will also fetch cookies.","ref":"Plug.Conn.html#fetch_session/2","title":"Plug.Conn.fetch_session/2","type":"function"},{"doc":"Returns the HTTP protocol and version.Examplesiex&gt; get_http_protocol(conn) :&quot;HTTP/1.1&quot;","ref":"Plug.Conn.html#get_http_protocol/1","title":"Plug.Conn.get_http_protocol/1","type":"function"},{"doc":"Returns the request peer data if one is present.","ref":"Plug.Conn.html#get_peer_data/1","title":"Plug.Conn.get_peer_data/1","type":"function"},{"doc":"Returns the values of the request header specified by key.Examplesiex&gt; get_req_header(conn, &quot;accept&quot;) [&quot;application/json&quot;]","ref":"Plug.Conn.html#get_req_header/2","title":"Plug.Conn.get_req_header/2","type":"function"},{"doc":"Returns the values of the response header specified by key.Examplesiex&gt; conn = %{conn | resp_headers: [{&quot;content-type&quot;, &quot;text/plain&quot;}]} iex&gt; get_resp_header(conn, &quot;content-type&quot;) [&quot;text/plain&quot;]","ref":"Plug.Conn.html#get_resp_header/2","title":"Plug.Conn.get_resp_header/2","type":"function"},{"doc":"Returns the whole session.Although get_session/2 and put_session/3 allow atom keys, they are always normalized to strings. So this function always returns a map with string keys.Raises if the session was not yet fetched.","ref":"Plug.Conn.html#get_session/1","title":"Plug.Conn.get_session/1","type":"function"},{"doc":"Returns session value for the given key. If key is not set, nil is returned.The key can be a string or an atom, where atoms are automatically converted to strings.","ref":"Plug.Conn.html#get_session/2","title":"Plug.Conn.get_session/2","type":"function"},{"doc":"Halts the Plug pipeline by preventing further plugs downstream from being invoked. See the docs for Plug.Builder for more information on halting a Plug pipeline.","ref":"Plug.Conn.html#halt/1","title":"Plug.Conn.halt/1","type":"function"},{"doc":"Sends and informational response to the client.An informational response, such as an early hint, must happen prior to a response being sent. If an informational request is attempted after a response is sent then a Plug.Conn.AlreadySentError will be raised. Only status codes from 100-199 are valid.To use inform for early hints send one or more informs with a status of 103.If the adapter does not support informational responses then this is a noop.Most HTTP/1.1 clients do not properly support informational responses but some proxies require it to support server push for HTTP/2. You can call get_http_protocol/1 to retrieve the protocol and version.","ref":"Plug.Conn.html#inform/3","title":"Plug.Conn.inform/3","type":"function"},{"doc":"Sends an information response to a client but raises if the adapter does not support inform.See inform/1 for more information.","ref":"Plug.Conn.html#inform!/3","title":"Plug.Conn.inform!/3","type":"function"},{"doc":"Assigns multiple values to keys in the connection.Equivalent to multiple calls to assign/3.Examplesiex&gt; conn.assigns[:hello] nil iex&gt; conn = merge_assigns(conn, hello: :world) iex&gt; conn.assigns[:hello] :world","ref":"Plug.Conn.html#merge_assigns/2","title":"Plug.Conn.merge_assigns/2","type":"function"},{"doc":"Assigns multiple private keys and values in the connection.Equivalent to multiple put_private/3 calls.Examplesiex&gt; conn.private[:my_plug_hello] nil iex&gt; conn = merge_private(conn, my_plug_hello: :world) iex&gt; conn.private[:my_plug_hello] :world","ref":"Plug.Conn.html#merge_private/2","title":"Plug.Conn.merge_private/2","type":"function"},{"doc":"Merges a series of response headers into the connection.It is recommended for header keys to be in lowercase, to avoid sending duplicate keys in a request. Additionally, responses with mixed-case headers served over HTTP/2 are not considered valid by common clients, resulting in dropped responses. As a convenience, when using the Plug.Adapters.Conn.Test adapter, any headers that aren't lowercase will raise a Plug.Conn.InvalidHeaderError.ExamplePlug.Conn.merge_resp_headers(conn, [{&quot;content-type&quot;, &quot;text/plain&quot;}, {&quot;X-1337&quot;, &quot;5P34K&quot;}])","ref":"Plug.Conn.html#merge_resp_headers/2","title":"Plug.Conn.merge_resp_headers/2","type":"function"},{"doc":"Prepends the list of headers to the connection response headers.Similar to put_resp_header this functions adds a new response header (key) but rather then replacing the existing one it prepends another header with the same key.It is recommended for header keys to be in lowercase, to avoid sending duplicate keys in a request. Additionally, responses with mixed-case headers served over HTTP/2 are not considered valid by common clients, resulting in dropped responses. As a convenience, when using the Plug.Adapters.Conn.Test adapter, any headers that aren't lowercase will raise a Plug.Conn.InvalidHeaderError.Raises a Plug.Conn.AlreadySentError if the connection has already been :sent or :chunked.Raises a Plug.Conn.InvalidHeaderError if the header value contains control feed (\\r) or newline (\\n) characters.ExamplesPlug.Conn.prepend_resp_headers(conn, [{&quot;content-type&quot;, &quot;application/json&quot;}])","ref":"Plug.Conn.html#prepend_resp_headers/2","title":"Plug.Conn.prepend_resp_headers/2","type":"function"},{"doc":"Pushes a resource to the client.Server pushes must happen prior to a response being sent. If a server push is attempted after a response is sent then a Plug.Conn.AlreadySentError will be raised.If the adapter does not support server push then this is a noop.Note that certain browsers (such as Google Chrome) will not accept a pushed resource if your certificate is not trusted. In the case of Chrome this means a valid cert with a SAN. See https://www.chromestatus.com/feature/4981025180483584","ref":"Plug.Conn.html#push/3","title":"Plug.Conn.push/3","type":"function"},{"doc":"Pushes a resource to the client but raises if the adapter does not support server push.","ref":"Plug.Conn.html#push!/3","title":"Plug.Conn.push!/3","type":"function"},{"doc":"Assigns a new private key and value in the connection.This storage is meant to be used by libraries and frameworks to avoid writing to the user storage (the :assigns field). It is recommended for libraries/frameworks to prefix the keys with the library name.For example, if a plug called my_plug needs to store a :hello key, it would store it as :my_plug_hello:iex&gt; conn.private[:my_plug_hello] nil iex&gt; conn = put_private(conn, :my_plug_hello, :world) iex&gt; conn.private[:my_plug_hello] :world","ref":"Plug.Conn.html#put_private/3","title":"Plug.Conn.put_private/3","type":"function"},{"doc":"Adds a new request header (key) if not present, otherwise replaces the previous value of that header with value.Because header keys are case-insensitive in both HTTP/1.1 and HTTP/2, it is recommended for header keys to be in lowercase, to avoid sending duplicate keys in a request. Additionally, requests with mixed-case headers served over HTTP/2 are not considered valid by common clients, resulting in dropped requests. As a convenience, when using the Plug.Adapters.Conn.Test adapter, any headers that aren't lowercase will raise a Plug.Conn.InvalidHeaderError.Raises a Plug.Conn.AlreadySentError if the connection has already been :sent or :chunked.ExamplesPlug.Conn.put_req_header(conn, &quot;accept&quot;, &quot;application/json&quot;)","ref":"Plug.Conn.html#put_req_header/3","title":"Plug.Conn.put_req_header/3","type":"function"},{"doc":"Sets the value of the &quot;content-type&quot; response header taking into account the charset.If charset is nil, the value of the &quot;content-type&quot; response header won't specify a charset.Examplesiex&gt; conn = put_resp_content_type(conn, &quot;application/json&quot;) iex&gt; get_resp_header(conn, &quot;content-type&quot;) [&quot;application/json; charset=utf-8&quot;]","ref":"Plug.Conn.html#put_resp_content_type/3","title":"Plug.Conn.put_resp_content_type/3","type":"function"},{"doc":"Puts a response cookie in the connection.If the :sign or :encrypt flag are given, then the cookie value can be any term.If the cookie is not signed nor encrypted, then the value must be a binary. Note the value is not automatically escaped. Therefore if you want to store values with non-alphanumeric characters, you must either sign or encrypt the cookie or consider explicitly escaping the cookie value by using a function such as Base.encode64(value, padding: false) when writing and Base.decode64(encoded, padding: false) when reading the cookie. It is important for padding to be disabled since = is not a valid character in cookie values.Signing and encrypting cookiesThis function allows you to automatically sign and encrypt cookies. When signing or encryption is enabled, then any Elixir value can be stored in the cookie (except anonymous functions for security reasons). Once a value is signed or encrypted, you must also call fetch_cookies/2 with the name of the cookies that are either signed or encrypted.To sign, you would do:put_resp_cookie(conn, &quot;my-cookie&quot;, %{user_id: user.id}, sign: true)and then:fetch_cookies(conn, signed: ~w(my-cookie))To encrypt, you would do:put_resp_cookie(conn, &quot;my-cookie&quot;, %{user_id: user.id}, encrypt: true)and then:fetch_cookies(conn, encrypted: ~w(my-cookie))By default a signed or encrypted cookie is only valid for a day, unless a :max_age is specified.The signing and encryption keys are derived from the connection's secret_key_base using a salt that is built by appending &quot;_cookie&quot; to the cookie name. Care should be taken not to derive other keys using this value as the salt. Similarly do not use the same cookie name to store different values with distinct purposes.Options:domain - the domain the cookie applies to:max_age - the cookie max-age, in seconds. Providing a value for this option will set both the max-age and expires cookie attributes.:path - the path the cookie applies to:http_only - when false, the cookie is accessible beyond HTTP:secure - if the cookie must be sent only over https. Defaults to true when the connection is HTTPS:extra - string to append to cookie. Use this to take advantage of non-standard cookie attributes.:sign - when true, signs the cookie:encrypt - when true, encrypts the cookie:same_site - set the cookie SameSite attribute to a string value. If no string value is set, the attribute is omitted.","ref":"Plug.Conn.html#put_resp_cookie/4","title":"Plug.Conn.put_resp_cookie/4","type":"function"},{"doc":"Adds a new response header (key) if not present, otherwise replaces the previous value of that header with value.Because header keys are case-insensitive in both HTTP/1.1 and HTTP/2, it is recommended for header keys to be in lowercase, to avoid sending duplicate keys in a request. Additionally, responses with mixed-case headers served over HTTP/2 are not considered valid by common clients, resulting in dropped responses. As a convenience, when using the Plug.Adapters.Conn.Test adapter, any headers that aren't lowercase will raise a Plug.Conn.InvalidHeaderError.Raises a Plug.Conn.AlreadySentError if the connection has already been :sent or :chunked.Raises a Plug.Conn.InvalidHeaderError if the header value contains control feed (\\r) or newline (\\n) characters.ExamplesPlug.Conn.put_resp_header(conn, &quot;content-type&quot;, &quot;application/json&quot;)","ref":"Plug.Conn.html#put_resp_header/3","title":"Plug.Conn.put_resp_header/3","type":"function"},{"doc":"Puts the specified value in the session for the given key.The key can be a string or an atom, where atoms are automatically converted to strings. Can only be invoked on unsent conns. Will raise otherwise.","ref":"Plug.Conn.html#put_session/3","title":"Plug.Conn.put_session/3","type":"function"},{"doc":"Stores the given status code in the connection.The status code can be nil, an integer, or an atom. The list of allowed atoms is available in Plug.Conn.Status.Raises a Plug.Conn.AlreadySentError if the connection has already been :sent or :chunked.ExamplesPlug.Conn.put_status(conn, :not_found) Plug.Conn.put_status(conn, 200)","ref":"Plug.Conn.html#put_status/2","title":"Plug.Conn.put_status/2","type":"function"},{"doc":"Reads the request body.This function reads a chunk of the request body up to a given length (specified by the :length option). If there is more data to be read, then {:more, partial_body, conn} is returned. Otherwise {:ok, body, conn} is returned. In case of an error reading the socket, {:error, reason} is returned as per :gen_tcp.recv/2.Like all functions in this module, the conn returned by read_body must be passed to the next stage of your pipeline and should not be ignored.In order to, for instance, support slower clients you can tune the :read_length and :read_timeout options. These specify how much time should be allowed to pass for each read from the underlying socket.Because the request body can be of any size, reading the body will only work once, as Plug will not cache the result of these operations. If you need to access the body multiple times, it is your responsibility to store it. Finally keep in mind some plugs like Plug.Parsers may read the body, so the body may be unavailable after being accessed by such plugs.This function is able to handle both chunked and identity transfer-encoding by default.Options:length - sets the maximum number of bytes to read from the body on every call, defaults to 8_000_000 bytes:read_length - sets the amount of bytes to read at one time from the underlying socket to fill the chunk, defaults to 1_000_000 bytes:read_timeout - sets the timeout for each socket read, defaults to 15_000 millisecondsThe values above are not meant to be exact. For example, setting the length to 8_000_000 may end up reading some hundred bytes more from the socket until we halt.Examples{:ok, body, conn} = Plug.Conn.read_body(conn, length: 1_000_000)","ref":"Plug.Conn.html#read_body/2","title":"Plug.Conn.read_body/2","type":"function"},{"doc":"Reads the body of a multipart request.Returns {:ok, body, conn} if all body has been read, {:more, binary, conn} otherwise, and {:done, conn} if there is no more body.It accepts the same options as read_body/2.","ref":"Plug.Conn.html#read_part_body/2","title":"Plug.Conn.read_part_body/2","type":"function"},{"doc":"Reads the headers of a multipart request.It returns {:ok, headers, conn} with the headers or {:done, conn} if there are no more parts.Once read_part_headers/2 is invoked, you may call read_part_body/2 to read the body associated to the headers. If read_part_headers/2 is called instead, the body is automatically skipped until the next part headers.Options:length - sets the maximum number of bytes to read from the body for each chunk, defaults to 64_000 bytes:read_length - sets the amount of bytes to read at one time from the underlying socket to fill the chunk, defaults to 64_000 bytes:read_timeout - sets the timeout for each socket read, defaults to 5_000 milliseconds","ref":"Plug.Conn.html#read_part_headers/2","title":"Plug.Conn.read_part_headers/2","type":"function"},{"doc":"Registers a callback to be invoked before the response is sent.Callbacks are invoked in the reverse order they are defined (callbacks defined first are invoked last).ExamplesTo log the status of response being sent:require Logger Plug.Conn.register_before_send(conn, fn conn -&gt; Logger.info(&quot;Sent a \#{conn.status} response&quot;) conn end)","ref":"Plug.Conn.html#register_before_send/2","title":"Plug.Conn.register_before_send/2","type":"function"},{"doc":"Returns the full request URL.","ref":"Plug.Conn.html#request_url/1","title":"Plug.Conn.request_url/1","type":"function"},{"doc":"Sets the response to the given status and body.It sets the connection state to :set (if not already :set) and raises Plug.Conn.AlreadySentError if it was already :sent.If you also want to send the response, use send_resp/1 after this or use send_resp/3.The status can be an integer, an atom, or nil. See Plug.Conn.Status for more information.ExamplesPlug.Conn.resp(conn, 404, &quot;Not found&quot;)","ref":"Plug.Conn.html#resp/3","title":"Plug.Conn.resp/3","type":"function"},{"doc":"Sends the response headers as a chunked response.It expects a connection that has not been :sent yet and sets its state to :chunked afterwards. Otherwise, raises Plug.Conn.AlreadySentError. After send_chunked/2 is called, chunks can be sent to the client via the chunk/2 function.HTTP/2 does not support chunking and will instead stream the response without a transfer encoding. When using HTTP/1.1, the Cowboy adapter will stream the response instead of emitting chunks if the content-length header has been set before calling send_chunked/2.","ref":"Plug.Conn.html#send_chunked/2","title":"Plug.Conn.send_chunked/2","type":"function"},{"doc":"Sends a file as the response body with the given status and optionally starting at the given offset until the given length.If available, the file is sent directly over the socket using the operating system sendfile operation.It expects a connection that has not been :sent yet and sets its state to :file afterwards. Otherwise raises Plug.Conn.AlreadySentError.ExamplesPlug.Conn.send_file(conn, 200, &quot;README.md&quot;)","ref":"Plug.Conn.html#send_file/5","title":"Plug.Conn.send_file/5","type":"function"},{"doc":"Sends a response to the client.It expects the connection state to be :set, otherwise raises an ArgumentError for :unset connections or a Plug.Conn.AlreadySentError for already :sent connections.At the end sets the connection state to :sent.Note that this function does not halt the connection, so if subsequent plugs try to send another response, it will error out. Use halt/1 after this function if you want to halt the plug pipeline.Examplesconn |&gt; Plug.Conn.resp(404, &quot;Not found&quot;) |&gt; Plug.Conn.send_resp()","ref":"Plug.Conn.html#send_resp/1","title":"Plug.Conn.send_resp/1","type":"function"},{"doc":"Sends a response with the given status and body.This is equivalent to setting the status and the body and then calling send_resp/1.Note that this function does not halt the connection, so if subsequent plugs try to send another response, it will error out. Use halt/1 after this function if you want to halt the plug pipeline.ExamplesPlug.Conn.send_resp(conn, 404, &quot;Not found&quot;)","ref":"Plug.Conn.html#send_resp/3","title":"Plug.Conn.send_resp/3","type":"function"},{"doc":"Updates a request header if present, otherwise it sets it to an initial value.Raises a Plug.Conn.AlreadySentError if the connection has already been :sent or :chunked.Only the first value of the header key is updated if present.ExamplesPlug.Conn.update_req_header( conn, &quot;accept&quot;, &quot;application/json; charset=utf-8&quot;, &amp;(&amp;1 &lt;&gt; &quot;; charset=utf-8&quot;) )","ref":"Plug.Conn.html#update_req_header/4","title":"Plug.Conn.update_req_header/4","type":"function"},{"doc":"Updates a response header if present, otherwise it sets it to an initial value.Raises a Plug.Conn.AlreadySentError if the connection has already been :sent or :chunked.Only the first value of the header key is updated if present.ExamplesPlug.Conn.update_resp_header( conn, &quot;content-type&quot;, &quot;application/json; charset=utf-8&quot;, &amp;(&amp;1 &lt;&gt; &quot;; charset=utf-8&quot;) )","ref":"Plug.Conn.html#update_resp_header/4","title":"Plug.Conn.update_resp_header/4","type":"function"},{"doc":"","ref":"Plug.Conn.html#t:adapter/0","title":"Plug.Conn.adapter/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:assigns/0","title":"Plug.Conn.assigns/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:before_send/0","title":"Plug.Conn.before_send/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:body/0","title":"Plug.Conn.body/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:cookies/0","title":"Plug.Conn.cookies/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:halted/0","title":"Plug.Conn.halted/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:headers/0","title":"Plug.Conn.headers/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:host/0","title":"Plug.Conn.host/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:int_status/0","title":"Plug.Conn.int_status/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:method/0","title":"Plug.Conn.method/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:owner/0","title":"Plug.Conn.owner/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:params/0","title":"Plug.Conn.params/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:port_number/0","title":"Plug.Conn.port_number/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:query_param/0","title":"Plug.Conn.query_param/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:query_params/0","title":"Plug.Conn.query_params/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:query_string/0","title":"Plug.Conn.query_string/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:req_cookies/0","title":"Plug.Conn.req_cookies/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:resp_cookies/0","title":"Plug.Conn.resp_cookies/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:scheme/0","title":"Plug.Conn.scheme/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:secret_key_base/0","title":"Plug.Conn.secret_key_base/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:segments/0","title":"Plug.Conn.segments/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:state/0","title":"Plug.Conn.state/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:status/0","title":"Plug.Conn.status/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:t/0","title":"Plug.Conn.t/0","type":"type"},{"doc":"Conveniences for generating HTML.","ref":"Plug.HTML.html","title":"Plug.HTML","type":"module"},{"doc":"Escapes the given HTML to string.iex&gt; Plug.HTML.html_escape(&quot;foo&quot;) &quot;foo&quot; iex&gt; Plug.HTML.html_escape(&quot;&lt;foo&gt;&quot;) &quot;&amp;lt;foo&amp;gt;&quot; iex&gt; Plug.HTML.html_escape(&quot;quotes: \\&quot; &amp; \\&#39;&quot;) &quot;quotes: &amp;quot; &amp;amp; &amp;#39;&quot;","ref":"Plug.HTML.html#html_escape/1","title":"Plug.HTML.html_escape/1","type":"function"},{"doc":"Escapes the given HTML to iodata.iex&gt; Plug.HTML.html_escape_to_iodata(&quot;foo&quot;) &quot;foo&quot; iex&gt; Plug.HTML.html_escape_to_iodata(&quot;&lt;foo&gt;&quot;) [[[] | &quot;&amp;lt;&quot;], &quot;foo&quot; | &quot;&amp;gt;&quot;] iex&gt; Plug.HTML.html_escape_to_iodata(&quot;quotes: \\&quot; &amp; \\&#39;&quot;) [[[[], &quot;quotes: &quot; | &quot;&amp;quot;&quot;], &quot; &quot; | &quot;&amp;amp;&quot;], &quot; &quot; | &quot;&amp;#39;&quot;]","ref":"Plug.HTML.html#html_escape_to_iodata/1","title":"Plug.HTML.html_escape_to_iodata/1","type":"function"},{"doc":"A DSL to define a routing algorithm that works with Plug.It provides a set of macros to generate routes. For example:defmodule AppRouter do use Plug.Router plug :match plug :dispatch get &quot;/hello&quot; do send_resp(conn, 200, &quot;world&quot;) end match _ do send_resp(conn, 404, &quot;oops&quot;) end endEach route receives a conn variable containing a Plug.Conn struct and it needs to return a connection, as per the Plug spec. A catch-all match is recommended to be defined as in the example above, otherwise routing fails with a function clause error.The router is itself a plug, which means it can be invoked as:AppRouter.call(conn, AppRouter.init([]))Each Plug.Router has a plug pipeline, defined by Plug.Builder, and by default it requires two plugs: :match and :dispatch. :match is responsible for finding a matching route which is then forwarded to :dispatch. This means users can easily hook into the router mechanism and add behaviour before match, before dispatch, or after both. All of the options given to use Plug.Router are forwarded to Plug.Builder. See the Plug.Builder module for more information on the plug macro and on the available options.Routesget &quot;/hello&quot; do send_resp(conn, 200, &quot;world&quot;) endIn the example above, a request will only match if it is a GET request and the route is &quot;/hello&quot;. The supported HTTP methods are get, post, put, patch, delete and options.A route can also specify parameters which will then be available in the function body:get &quot;/hello/:name&quot; do send_resp(conn, 200, &quot;hello \#{name}&quot;) endThe :name parameter will also be available in the function body as conn.params[&quot;name&quot;] and conn.path_params[&quot;name&quot;].Routes allow for globbing which will match the remaining parts of a route and can be available as a parameter in the function body. Also note that a glob can't be followed by other segments:get &quot;/hello/*_rest&quot; do send_resp(conn, 200, &quot;matches all routes starting with /hello&quot;) end get &quot;/hello/*glob&quot; do send_resp(conn, 200, &quot;route after /hello: \#{inspect glob}&quot;) endFinally, a general match function is also supported:match &quot;/hello&quot; do send_resp(conn, 200, &quot;world&quot;) endA match will match any route regardless of the HTTP method. Check match/3 for more information on how route compilation works and a list of supported options.Parameter ParsingHandling request data can be done through the Plug.Parsers plug. It provides support for parsing URL-encoded, form-data, and JSON data as well as providing a behaviour that others parsers can adopt.Here is an example of Plug.Parsers can be used in a Plug.Router router to parse the JSON-encoded body of a POST request:defmodule AppRouter do use Plug.Router plug :match plug Plug.Parsers, parsers: [:json], pass: [&quot;application/json&quot;], json_decoder: Jason plug :dispatch post &quot;/hello&quot; do IO.inspect conn.body_params # Prints JSON POST body send_resp(conn, 200, &quot;Success!&quot;) end endIt is important that Plug.Parsers is placed before the :dispatch plug in the pipeline, otherwise the matched clause route will not receive the parsed body in its Plug.Conn argument when dispatched.Plug.Parsers can also be plugged between :match and :dispatch (like in the example above): this means that Plug.Parsers will run only if there is a matching route. This can be useful to perform actions such as authentication before parsing the body, which should only be parsed if a route matches afterwards.Error handlingIn case something goes wrong in a request, the router by default will crash, without returning any response to the client. This behaviour can be configured in two ways, by using two different modules:Plug.ErrorHandler - allows the developer to customize exactly which page is sent to the client via the handle_errors/2 function;Plug.Debugger - automatically shows debugging and request information about the failure. This module is recommended to be used only in a development environment.Here is an example of how both modules could be used in an application:defmodule AppRouter do use Plug.Router if Mix.env == :dev do use Plug.Debugger end use Plug.ErrorHandler plug :match plug :dispatch get &quot;/hello&quot; do send_resp(conn, 200, &quot;world&quot;) end defp handle_errors(conn, %{kind: _kind, reason: _reason, stack: _stack}) do send_resp(conn, conn.status, &quot;Something went wrong&quot;) end endPassing data between routes and plugsIt is also possible to assign data to the Plug.Conn that will be available to any plug invoked after the :match plug. This is very useful if you want a matched route to customize how later plugs will behave.You can use :assigns (which contains user data) or :private (which contains library/framework data) for this. For example:get &quot;/hello&quot;, assigns: %{an_option: :a_value} do send_resp(conn, 200, &quot;world&quot;) endIn the example above, conn.assigns[:an_option] will be available to all plugs invoked after :match. Such plugs can read from conn.assigns (or conn.private) to configure their behaviour based on the matched route.Routes compilationAll routes are compiled to a match function that receives three arguments: the method, the request path split on / and the connection. Consider this example:match &quot;/foo/bar&quot;, via: :get do send_resp(conn, 200, &quot;hello world&quot;) endIt is compiled to:defp match(&quot;GET&quot;, [&quot;foo&quot;, &quot;bar&quot;], conn) do send_resp(conn, 200, &quot;hello world&quot;) endThis means guards can be given to match:match &quot;/foo/bar/:baz&quot; when size(baz) &lt;= 3, via: :get do send_resp(conn, 200, &quot;hello world&quot;) endAfter a match is found, the block given as do/end is stored as a function in the connection. This function is then retrieved and invoked in the dispatch plug.Routes optionsSometimes you may want to customize how a route behaves during dispatch. This can be done by accessing the opts variable inside the route:defmodule AppRouter do use Plug.Router plug :match plug :dispatch, content: &quot;hello world&quot; get &quot;/hello&quot; do send_resp(conn, 200, opts[:content]) end match _ do send_resp(conn, 404, &quot;oops&quot;) end endThis is particularly useful when used with Plug.Builder.builder_opts/0. builder_opts/0 allows us to pass options received when initializing AppRouter to a specific plug, such as dispatch itself. So if instead of:plug :dispatch, content: &quot;hello world&quot;we do:plug :dispatch, builder_opts()now the content can be given when starting the router, like this:Plug.Cowboy.http AppRouter, [content: &quot;hello world&quot;]Or as part of a pipeline like this:plug AppRouter, content: &quot;hello world&quot;In a nutshell, builder_opts() allows us to pass the options given when initializing the router to a dispatch.TelemetryThe router emits the following telemetry events:[:plug, :router_dispatch, :start] - dispatched before dispatching to a matched routeMeasurement: %{system_time: System.system_time}Metadata: %{conn: Plug.Conn.t, route: binary, router: module}[:plug, :router_dispatch, :exception] - dispatched after exceptions on dispatching a routeMeasurement: %{duration: native_time}Metadata: %{conn: Plug.Conn.t, route: binary, router: module}[:plug, :router_dispatch, :stop] - dispatched after successfully dispatching a matched routeMeasurement: %{duration: native_time}Metadata: %{conn: Plug.Conn.t, route: binary, router: module}","ref":"Plug.Router.html","title":"Plug.Router","type":"module"},{"doc":"Dispatches to the path only if the request is a DELETE request. See match/3 for more examples.","ref":"Plug.Router.html#delete/3","title":"Plug.Router.delete/3","type":"macro"},{"doc":"Forwards requests to another Plug. The path_info of the forwarded connection will exclude the portion of the path specified in the call to forward. If the path contains any parameters, those will be available in the target Plug in conn.params and conn.path_params.Optionsforward accepts the following options::to - a Plug the requests will be forwarded to.:init_opts - the options for the target Plug.:host - a string representing the host or subdomain, exactly like in match/3.:private - values for conn.private, exactly like in match/3.:assigns - values for conn.assigns, exactly like in match/3.If :init_opts is undefined, then all remaining options are passed to the target plug.Examplesforward &quot;/users&quot;, to: UserRouterAssuming the above code, a request to /users/sign_in will be forwarded to the UserRouter plug, which will receive what it will see as a request to /sign_in.forward &quot;/foo/:bar/qux&quot;, to: FooPlugHere, a request to /foo/BAZ/qux will be forwarded to the FooPlug plug, which will receive what it will see as a request to /, and conn.params[&quot;bar&quot;] will be set to &quot;BAZ&quot;.Some other examples:forward &quot;/foo/bar&quot;, to: :foo_bar_plug, host: &quot;foobar.&quot; forward &quot;/baz&quot;, to: BazPlug, init_opts: [plug_specific_option: true]","ref":"Plug.Router.html#forward/2","title":"Plug.Router.forward/2","type":"macro"},{"doc":"Dispatches to the path only if the request is a GET request. See match/3 for more examples.","ref":"Plug.Router.html#get/3","title":"Plug.Router.get/3","type":"macro"},{"doc":"Dispatches to the path only if the request is a HEAD request. See match/3 for more examples.","ref":"Plug.Router.html#head/3","title":"Plug.Router.head/3","type":"macro"},{"doc":"Main API to define routes.It accepts an expression representing the path and many options allowing the match to be configured.The route can dispatch either to a function body or a Plug module.Examplesmatch &quot;/foo/bar&quot;, via: :get do send_resp(conn, 200, &quot;hello world&quot;) end match &quot;/baz&quot;, to: MyPlug, init_opts: [an_option: :a_value]Optionsmatch/3 and the other route macros accept the following options::host - the host which the route should match. Defaults to nil, meaning no host match, but can be a string like &quot;example.com&quot; or a string ending with &quot;.&quot;, like &quot;subdomain.&quot; for a subdomain match.:private - assigns values to conn.private for use in the match:assigns - assigns values to conn.assigns for use in the match:via - matches the route against some specific HTTP method(s) specified as an atom, like :get or :put, or a list, like [:get, :post].:do - contains the implementation to be invoked in case the route matches.:to - a Plug that will be called in case the route matches.:init_opts - the options for the target Plug given by :to.A route should specify only one of :do or :to options.","ref":"Plug.Router.html#match/3","title":"Plug.Router.match/3","type":"macro"},{"doc":"Returns the path of the route that the request was matched to.","ref":"Plug.Router.html#match_path/1","title":"Plug.Router.match_path/1","type":"function"},{"doc":"Dispatches to the path only if the request is an OPTIONS request. See match/3 for more examples.","ref":"Plug.Router.html#options/3","title":"Plug.Router.options/3","type":"macro"},{"doc":"Dispatches to the path only if the request is a PATCH request. See match/3 for more examples.","ref":"Plug.Router.html#patch/3","title":"Plug.Router.patch/3","type":"macro"},{"doc":"Dispatches to the path only if the request is a POST request. See match/3 for more examples.","ref":"Plug.Router.html#post/3","title":"Plug.Router.post/3","type":"macro"},{"doc":"Dispatches to the path only if the request is a PUT request. See match/3 for more examples.","ref":"Plug.Router.html#put/3","title":"Plug.Router.put/3","type":"macro"},{"doc":"Conveniences for testing plugs.This module can be used in your test cases, like this:use ExUnit.Case, async: true use Plug.TestUsing this module will:import all the functions from this moduleimport all the functions from the Plug.Conn moduleBy default, Plug tests checks for invalid header keys, e.g. header keys which include uppercase letters, and raises a Plug.Conn.InvalidHeaderError when it finds one. To disable it, set :validate_header_keys_during_test to false on the app config.config :plug, :validate_header_keys_during_test, false","ref":"Plug.Test.html","title":"Plug.Test","type":"module"},{"doc":"Creates a test connection.The request method and path are required arguments. method may be any value that implements to_string/1 and it will properly converted and normalized (e.g., :get or &quot;post&quot;).The path is commonly the request path with optional query string but it may also be a complete URI. When a URI is given, the host and schema will be used as part of the request too.The params_or_body field must be one of:nil - meaning there is no body;a binary - containing a request body. For such cases, :headers must be given as option with a content-type;a map or list - containing the parameters which will automatically set the content-type to multipart. The map or list may contain other lists or maps and all entries will be normalized to string keys;Examplesconn(:get, &quot;/foo?bar=10&quot;) conn(:get, &quot;/foo&quot;, %{bar: 10}) conn(:post, &quot;/&quot;) conn(&quot;patch&quot;, &quot;/&quot;, &quot;&quot;) |&gt; put_req_header(&quot;content-type&quot;, &quot;application/json&quot;)","ref":"Plug.Test.html#conn/3","title":"Plug.Test.conn/3","type":"function"},{"doc":"Deletes a request cookie.","ref":"Plug.Test.html#delete_req_cookie/2","title":"Plug.Test.delete_req_cookie/2","type":"function"},{"doc":"Initializes the session with the given contents.If the session has already been initialized, the new contents will be merged with the previous ones.","ref":"Plug.Test.html#init_test_session/2","title":"Plug.Test.init_test_session/2","type":"function"},{"doc":"Puts the http protocol.","ref":"Plug.Test.html#put_http_protocol/2","title":"Plug.Test.put_http_protocol/2","type":"function"},{"doc":"Puts the peer data.","ref":"Plug.Test.html#put_peer_data/2","title":"Plug.Test.put_peer_data/2","type":"function"},{"doc":"Puts a request cookie.","ref":"Plug.Test.html#put_req_cookie/3","title":"Plug.Test.put_req_cookie/3","type":"function"},{"doc":"Moves cookies from a connection into a new connection for subsequent requests.This function copies the cookie information in old_conn into new_conn, emulating multiple requests done by clients where cookies are always passed forward, and returns the new version of new_conn.","ref":"Plug.Test.html#recycle_cookies/2","title":"Plug.Test.recycle_cookies/2","type":"function"},{"doc":"Return the informational requests that have been sent.This function depends on gathering the messages sent by the test adapter when informational messages, such as an early hint, are sent. Calling this function will clear the informational request messages from the inbox for the process. To assert on multiple informs, the result of the function should be stored in a variable.Examplesconn = conn(:get, &quot;/foo&quot;, &quot;bar=10&quot;) informs = Plug.Test.sent_informs(conn) assert {&quot;/static/application.css&quot;, [{&quot;accept&quot;, &quot;text/css&quot;}]} in informs assert {&quot;/static/application.js&quot;, [{&quot;accept&quot;, &quot;application/javascript&quot;}]} in informs","ref":"Plug.Test.html#sent_informs/1","title":"Plug.Test.sent_informs/1","type":"function"},{"doc":"Return the assets that have been pushed.This function depends on gathering the messages sent by the test adapter when assets are pushed. Calling this function will clear the pushed message from the inbox for the process. To assert on multiple pushes, the result of the function should be stored in a variable.Examplesconn = conn(:get, &quot;/foo?bar=10&quot;) pushes = Plug.Test.sent_pushes(conn) assert {&quot;/static/application.css&quot;, [{&quot;accept&quot;, &quot;text/css&quot;}]} in pushes assert {&quot;/static/application.js&quot;, [{&quot;accept&quot;, &quot;application/javascript&quot;}]} in pushes","ref":"Plug.Test.html#sent_pushes/1","title":"Plug.Test.sent_pushes/1","type":"function"},{"doc":"Returns the sent response.This function is useful when the code being invoked crashes and there is a need to verify a particular response was sent even with the crash. It returns a tuple with {status, headers, body}.","ref":"Plug.Test.html#sent_resp/1","title":"Plug.Test.sent_resp/1","type":"function"},{"doc":"A server (a GenServer specifically) that manages uploaded files.Uploaded files are stored in a temporary directory and removed from that directory after the process that requested the file dies.During the request, files are represented with a Plug.Upload struct that contains three fields::path - the path to the uploaded file on the filesystem:content_type - the content type of the uploaded file:filename - the filename of the uploaded file given in the requestNote: as mentioned in the documentation for Plug.Parsers, the :plug application has to be started in order to upload files and use the Plug.Upload module.SecurityThe :content_type and :filename fields in the Plug.Upload struct are client-controlled. These values should be validated, via file content inspection or similar, before being trusted.","ref":"Plug.Upload.html","title":"Plug.Upload","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Plug.Upload.html#child_spec/1","title":"Plug.Upload.child_spec/1","type":"function"},{"doc":"Assign ownership of the given upload file to another process.Useful if you want to do some work on an uploaded file in another process since it means that the file will survive the end of the request.","ref":"Plug.Upload.html#give_away/3","title":"Plug.Upload.give_away/3","type":"function"},{"doc":"Requests a random file to be created in the upload directory with the given prefix.","ref":"Plug.Upload.html#random_file/1","title":"Plug.Upload.random_file/1","type":"function"},{"doc":"Requests a random file to be created in the upload directory with the given prefix. Raises on failure.","ref":"Plug.Upload.html#random_file!/1","title":"Plug.Upload.random_file!/1","type":"function"},{"doc":"","ref":"Plug.Upload.html#t:t/0","title":"Plug.Upload.t/0","type":"type"},{"doc":"Functionality for providing Basic HTTP authentication.It is recommended to only use this module in production if SSL is enabled and enforced. See Plug.SSL for more information.Compile-time usageIf you have a single username and password, you can use the basic_auth/2 plug:import Plug.BasicAuth plug :basic_auth, username: &quot;hello&quot;, password: &quot;secret&quot;Or if you would rather put those in a config file:# lib/your_app.ex import Plug.BasicAuth plug :basic_auth, Application.compile_env(:my_app, :basic_auth) # config/config.exs config :my_app, :basic_auth, username: &quot;hello&quot;, password: &quot;secret&quot;Once the user first accesses the page, the request will be denied with reason 401 and the request is halted. The browser will then prompt the user for username and password. If they match, then the request succeeds.Both approaches shown above rely on static configuration. Let's see alternatives.Runtime-time usageAs any other Plug, we can use the basic_auth at runtime by simply wrapping it in a function:plug :auth defp auth(conn, opts) do username = System.fetch_env!(&quot;AUTH_USERNAME&quot;) password = System.fetch_env!(&quot;AUTH_PASSWORD&quot;) Plug.BasicAuth.basic_auth(conn, username: username, password: password) endThis approach is useful when both username and password are specified upfront and available at runtime. However, you may also want to compute a different password for each different user. In those cases, we can use the low-level API.Low-level usageIf you want to provide your own authentication logic on top of Basic HTTP auth, you can use the low-level functions. As an example, we define :auth plug that extracts username and password from the request headers, compares them against the database, and either assigns a :current_user on success or responds with an error on failure.plug :auth defp auth(conn, _opts) do with {user, pass} &lt;- Plug.BasicAuth.parse_basic_auth(conn), %User{} = user &lt;- MyApp.Accounts.find_by_username_and_password(user, pass) do assign(conn, :current_user, user) else _ -&gt; conn |&gt; Plug.BasicAuth.request_basic_auth() |&gt; halt() end endKeep in mind that:The supplied user and pass may be empty strings;If you are comparing the username and password with existing strings, do not use ==/2. Use Plug.Crypto.secure_compare/2 instead.","ref":"Plug.BasicAuth.html","title":"Plug.BasicAuth","type":"module"},{"doc":"Higher level usage of Basic HTTP auth.See the module docs for examples.Options:username - the expected username:password - the expected password:realm - the authentication realm. The value is not fully sanitized, so do not accept user input as the realm and use strings with only alphanumeric characters and space","ref":"Plug.BasicAuth.html#basic_auth/2","title":"Plug.BasicAuth.basic_auth/2","type":"function"},{"doc":"Encodes a basic authentication header.This can be used during tests:put_req_header(conn, &quot;authorization&quot;, encode_basic_auth(&quot;hello&quot;, &quot;world&quot;))","ref":"Plug.BasicAuth.html#encode_basic_auth/2","title":"Plug.BasicAuth.encode_basic_auth/2","type":"function"},{"doc":"Parses the request username and password from Basic HTTP auth.It returns either {user, pass} or :error. Note the username and password may be empty strings. When comparing the username and password with the expected values, be sure to use Plug.Crypto.secure_compare/2.See the module docs for examples.","ref":"Plug.BasicAuth.html#parse_basic_auth/1","title":"Plug.BasicAuth.parse_basic_auth/1","type":"function"},{"doc":"Requests basic authentication from the client.It sets the response to status 401 with &quot;Unauthorized&quot; as body. The response is not sent though (nor the connection is halted), allowing developers to further customize it.Options:realm - the authentication realm. The value is not fully sanitized, so do not accept user input as the realm and use strings with only alphanumeric characters and space","ref":"Plug.BasicAuth.html#request_basic_auth/2","title":"Plug.BasicAuth.request_basic_auth/2","type":"function"},{"doc":"Plug to protect from cross-site request forgery.For this plug to work, it expects a session to have been previously fetched. It will then compare the token stored in the session with the one sent by the request to determine the validity of the request. For an invalid request the action taken is based on the :with option.The token may be sent by the request either via the params with key &quot;_csrf_token&quot; or a header with name &quot;x-csrf-token&quot;.GET requests are not protected, as they should not have any side-effect or change your application state. JavaScript requests are an exception: by using a script tag, external websites can embed server-side generated JavaScript, which can leak information. For this reason, this plug also forbids any GET JavaScript request that is not XHR (or AJAX).Note that it is recommended to enable CSRFProtection whenever a session is used, even for JSON requests. For example, Chrome had a bug that allowed POST requests to be triggered with arbitrary content-type, making JSON exploitable. More info: https://bugs.chromium.org/p/chromium/issues/detail?id=490015Finally, we recommend developers to invoke delete_csrf_token/0 every time after they log a user in, to avoid CSRF fixation attacks.Token generationThis plug won't generate tokens automatically. Instead, tokens will be generated only when required by calling get_csrf_token/0. In case you are generating the token for certain specific URL, you should use get_csrf_token_for/1 as that will avoid tokens from being leaked to other applications.Once a token is generated, it is cached in the process dictionary. The CSRF token is usually generated inside forms which may be isolated from Plug.Conn. Storing them in the process dictionary allows them to be generated as a side-effect only when necessary, becoming one of those rare situations where using the process dictionary is useful.Cross-host protectionIf you are sending data to a full URI, such as //subdomain.host.com/path or //external.com/path, instead of a simple path such as /path, you may want to consider using get_csrf_token_for/1, as that will encode the host in the CSRF token. Once received, Plug will only consider the CSRF token to be valid if the host encoded in the token is the same as the one in conn.host.Therefore, if you get a warning that the host does not match, it is either because someone is attempting to steal CSRF tokens or because you have a misconfigured host configuration.For example, if you are running your application behind a proxy, the browser will send a request to the proxy with www.example.com but the proxy will request you using an internal IP. In such cases, it is common for proxies to attach information such as &quot;x-forwarded-host&quot; that contains the original host.This may also happen on redirects. If you have a POST request to foo.example.com that redirects to bar.example.com with status 307, the token will contain a different host than the one in the request.You can pass the :allow_hosts option to control any host that you may want to allow. The values in :allow_hosts may either be a full host name or a host suffix. For example: [&quot;www.example.com&quot;, &quot;.subdomain.example.com&quot;] will allow the exact host of &quot;www.example.com&quot; and any host that ends with &quot;.subdomain.example.com&quot;.Options:session_key - the name of the key in session to store the token under:allow_hosts - a list with hosts to allow on cross-host tokens:with - should be one of :exception or :clear_session. Defaults to :exception.:exception - for invalid requests, this plug will raise Plug.CSRFProtection.InvalidCSRFTokenError.:clear_session - for invalid requests, this plug will set an empty session for only this request. Also any changes to the session during this request will be ignored.DisablingYou may disable this plug by doing Plug.Conn.put_private(conn, :plug_skip_csrf_protection, true). This was made available for disabling Plug.CSRFProtection in tests and not for dynamically skipping Plug.CSRFProtection in production code. If you want specific routes to skip Plug.CSRFProtection, then use a different stack of plugs for that route that does not include Plug.CSRFProtection.Examplesplug Plug.Session, ... plug :fetch_session plug Plug.CSRFProtection","ref":"Plug.CSRFProtection.html","title":"Plug.CSRFProtection","type":"module"},{"doc":"Deletes the CSRF token from the process dictionary.This will force the token to be deleted once the response is sent.","ref":"Plug.CSRFProtection.html#delete_csrf_token/0","title":"Plug.CSRFProtection.delete_csrf_token/0","type":"function"},{"doc":"Dump CSRF state so it can be loaded in another process.This function must be called after the Plug.CSRFProtection plug is invoked. If a token was not yet computed, it will be.See load_state/2 for more information.","ref":"Plug.CSRFProtection.html#dump_state/0","title":"Plug.CSRFProtection.dump_state/0","type":"function"},{"doc":"Dumps the CSRF state from the connection.It expects the value of get_session(conn, &quot;_csrf_token&quot;). It returns nil if there is no state in the session.","ref":"Plug.CSRFProtection.html#dump_state_from_session/1","title":"Plug.CSRFProtection.dump_state_from_session/1","type":"function"},{"doc":"Gets the CSRF token.Generates a token and stores it in the process dictionary if one does not exist.","ref":"Plug.CSRFProtection.html#get_csrf_token/0","title":"Plug.CSRFProtection.get_csrf_token/0","type":"function"},{"doc":"Gets the CSRF token for the associated URL (as a string or a URI struct).If the URL has a host, a CSRF token that is tied to that host will be generated. If it is a relative path URL, a simple token emitted with get_csrf_token/0 will be used.","ref":"Plug.CSRFProtection.html#get_csrf_token_for/1","title":"Plug.CSRFProtection.get_csrf_token_for/1","type":"function"},{"doc":"Load CSRF state into the process dictionary.This can be used to load CSRF state into another process. See dump_state/0 and dump_state_from_session/2 for dumping it.ExamplesTo dump the state from the current process and load into another one:csrf_state = Plug.CSRFProtection.dump_state() secret_key_base = conn.secret_key_base Task.async(fn -&gt; Plug.CSRFProtection.load_state(secret_key_base, csrf_state) end)If you have a session but the CSRF state was not loaded into the current process, you can dump the state from the session:csrf_state = Plug.CSRFProtection.dump_state_from_session(session[&quot;_csrf_token&quot;]) Task.async(fn -&gt; Plug.CSRFProtection.load_state(secret_key_base, csrf_state) end)","ref":"Plug.CSRFProtection.html#load_state/2","title":"Plug.CSRFProtection.load_state/2","type":"function"},{"doc":"Validates the state is valid against the given csrf_token.","ref":"Plug.CSRFProtection.html#valid_state_and_csrf_token?/2","title":"Plug.CSRFProtection.valid_state_and_csrf_token?/2","type":"function"},{"doc":"A Plug to convert HEAD requests to GET requests.ExamplesPlug.Head.call(conn, [])","ref":"Plug.Head.html","title":"Plug.Head","type":"module"},{"doc":"A plug for logging basic request information in the format:GET /index.html Sent 200 in 572msTo use it, just plug it into the desired module.plug Plug.Logger, log: :debugOptions:log - The log level at which this plug should log its request info. Default is :info.","ref":"Plug.Logger.html","title":"Plug.Logger","type":"module"},{"doc":"This plug overrides the request's POST method with the method defined in the _method request parameter.The POST method can be overridden only by these HTTP methods:PUTPATCHDELETEThis plug expects the body parameters to be already parsed and fetched. Those can be fetched with Plug.Parsers.This plug doesn't accept any options.ExamplesPlug.MethodOverride.call(conn, [])","ref":"Plug.MethodOverride.html","title":"Plug.MethodOverride","type":"module"},{"doc":"A plug for parsing the request body.It invokes a list of :parsers, which are activated based on the request content-type. Custom parsers are also supported by defining a module that implements the behaviour defined by this module.Once a connection goes through this plug, it will have :body_params set to the map of params parsed by one of the parsers listed in :parsers and :params set to the result of merging the :body_params and :query_params. In case :query_params have not yet been parsed, Plug.Conn.fetch_query_params/2 is automatically invoked.This plug will raise Plug.Parsers.UnsupportedMediaTypeError by default if the request cannot be parsed by any of the given types and the MIME type has not been explicitly accepted with the :pass option.Plug.Parsers.RequestTooLargeError will be raised if the request goes over the given limit. The default length is 8MB and it can be customized by passing the :length option to the Plug. :read_timeout and :read_length, as described by Plug.Conn.read_body/2, are also supported.Parsers may raise a Plug.Parsers.ParseError if the request has a malformed body.This plug only parses the body if the request method is one of the following:POSTPUTPATCHDELETEFor requests with a different request method, this plug will only fetch the query params.Options:parsers - a list of modules or atoms of built-in parsers to be invoked for parsing. These modules need to implement the behaviour outlined in this module.:pass - an optional list of MIME type strings that are allowed to pass through. Any mime not handled by a parser and not explicitly listed in :pass will raise UnsupportedMediaTypeError. For example:[&quot;*/*&quot;] - never raises[&quot;text/html&quot;, &quot;application/*&quot;] - doesn't raise for those values[] - always raises (default):query_string_length - the maximum allowed size for query strings:validate_utf8 - boolean that tells whether or not we want to validate that parsed binaries are utf8 strings.:body_reader - an optional replacement (or wrapper) for Plug.Conn.read_body/2 to provide a function that gives access to the raw body before it is parsed and discarded. It is in the standard format of {Module, :function, [args]} (MFA) and defaults to {Plug.Conn, :read_body, []}. Note that this option is not used by Plug.Parsers.MULTIPART which relies instead on other functions defined in Plug.Conn.All other options given to this Plug are forwarded to the parsers.Examplesplug Plug.Parsers, parsers: [:urlencoded, :multipart], pass: [&quot;text/*&quot;]Any other option given to Plug.Parsers is forwarded to the underlying parsers. Therefore, you can use a JSON parser and pass the :json_decoder option at the root:plug Plug.Parsers, parsers: [:urlencoded, :json], json_decoder: JasonOr directly to the parser itself:plug Plug.Parsers, parsers: [:urlencoded, {:json, json_decoder: Jason}]A common set of shared options given to Plug.Parsers is :length, :read_length and :read_timeout, which customizes the maximum request length you want to accept. For example, to support file uploads, you can do:plug Plug.Parsers, parsers: [:url_encoded, :multipart], length: 20_000_000However, the above will increase the maximum length of all request types. If you want to increase the limit only for multipart requests (which is typically the ones used for file uploads), you can do:plug Plug.Parsers, parsers: [ :url_encoded, {:multipart, length: 20_000_000} # Increase to 20MB max upload ]Built-in parsersPlug ships with the following parsers:Plug.Parsers.URLENCODED - parses application/x-www-form-urlencoded requests (can be used as :urlencoded as well in the :parsers option)Plug.Parsers.MULTIPART - parses multipart/form-data and multipart/mixed requests (can be used as :multipart as well in the :parsers option)Plug.Parsers.JSON - parses application/json requests with the given :json_decoder (can be used as :json as well in the :parsers option)File handlingIf a file is uploaded via any of the parsers, Plug will stream the uploaded contents to a file in a temporary directory in order to avoid loading the whole file into memory. For such, the :plug application needs to be started in order for file uploads to work. More details on how the uploaded file is handled can be found in the documentation for Plug.Upload.When a file is uploaded, the request parameter that identifies that file will be a Plug.Upload struct with information about the uploaded file (e.g. filename and content type) and about where the file is stored.The temporary directory where files are streamed to can be customized by setting the PLUG_TMPDIR environment variable on the host system. If PLUG_TMPDIR isn't set, Plug will look at some environment variables which usually hold the value of the system's temporary directory (like TMPDIR or TMP). If no value is found in any of those variables, /tmp is used as a default.Custom body readerSometimes you may want to customize how a parser reads the body from the connection. For example, you may want to cache the body to perform verification later, such as HTTP Signature Verification. This can be achieved with a custom body reader that would read the body and store it in the connection, such as:defmodule CacheBodyReader do def read_body(conn, opts) do {:ok, body, conn} = Plug.Conn.read_body(conn, opts) conn = update_in(conn.assigns[:raw_body], &amp;[body | (&amp;1 || [])]) {:ok, body, conn} end endwhich could then be set as:plug Plug.Parsers, parsers: [:urlencoded, :json], pass: [&quot;text/*&quot;], body_reader: {CacheBodyReader, :read_body, []}, json_decoder: Jason","ref":"Plug.Parsers.html","title":"Plug.Parsers","type":"behaviour"},{"doc":"","ref":"Plug.Parsers.html#c:init/1","title":"Plug.Parsers.init/1","type":"callback"},{"doc":"Attempts to parse the connection's request body given the content-type type, subtype, and its parameters.The arguments are:the Plug.Conn connectiontype, the content-type type (e.g., &quot;x-sample&quot; for the &quot;x-sample/json&quot; content-type)subtype, the content-type subtype (e.g., &quot;json&quot; for the &quot;x-sample/json&quot; content-type)params, the content-type parameters (e.g., %{&quot;foo&quot; =&gt; &quot;bar&quot;} for the &quot;text/plain; foo=bar&quot; content-type)This function should return:{:ok, body_params, conn} if the parser is able to handle the given content-type; body_params should be a map{:next, conn} if the next parser should be invoked{:error, :too_large, conn} if the request goes over the given limit","ref":"Plug.Parsers.html#c:parse/5","title":"Plug.Parsers.parse/5","type":"callback"},{"doc":"A plug for generating a unique request id for each request.The generated request id will be in the format &quot;uq8hs30oafhj5vve8ji5pmp7mtopc08f&quot;.If a request id already exists as the &quot;x-request-id&quot; HTTP request header, then that value will be used assuming it is between 20 and 200 characters. If it is not, a new request id will be generated.The request id is added to the Logger metadata as :request_id and the response as the &quot;x-request-id&quot; HTTP header. To see the request id in your log output, configure your logger backends to include the :request_id metadata:config :logger, :console, metadata: [:request_id]It is recommended to include this metadata configuration in your production configuration file.You can also access the request_id programmatically by calling Logger.metadata[:request_id]. Do not access it via the request header, as the request header value has not been validated and it may not always be present.To use this plug, just plug it into the desired module:plug Plug.RequestIdOptions:http_header - The name of the HTTP request header to check for existing request ids. This is also the HTTP response header that will be set with the request id. Default value is &quot;x-request-id&quot;plug Plug.RequestId, http_header: &quot;custom-request-id&quot;","ref":"Plug.RequestId.html","title":"Plug.RequestId","type":"module"},{"doc":"A plug to rewrite the request's host/port/protocol from x-forwarded-* headers.If your Plug application is behind a proxy that handles HTTPS, you may need to tell Plug to parse the proper protocol from the x-forwarded-* header.plug Plug.RewriteOn, [:x_forwarded_host, :x_forwarded_port, :x_forwarded_proto]The supported values are::x_forwarded_host - to override the host based on on the &quot;x-forwarded-host&quot; header:x_forwarded_port - to override the port based on on the &quot;x-forwarded-port&quot; header:x_forwarded_proto - to override the protocol based on on the &quot;x-forwarded-proto&quot; headerSince rewriting the scheme based on x-forwarded-* headers can open up security vulnerabilities, only use this plug if:your app is behind a proxyyour proxy strips the given x-forwarded-* headers from all incoming requestsyour proxy sets the x-forwarded-* headers and sends it to Plug","ref":"Plug.RewriteOn.html","title":"Plug.RewriteOn","type":"module"},{"doc":"A plug to force SSL connections and enable HSTS.If the scheme of a request is https, it'll add a strict-transport-security header to enable HTTP Strict Transport Security by default.Otherwise, the request will be redirected to a corresponding location with the https scheme by setting the location header of the response. The status code will be 301 if the method of conn is GET or HEAD, or 307 in other situations.Besides being a Plug, this module also provides conveniences for configuring SSL. See configure/1.x-forwarded-*If your Plug application is behind a proxy that handles HTTPS, you may need to tell Plug to parse the proper protocol from the x-forwarded-* header. This can be done using the :rewrite_on option:plug Plug.SSL, rewrite_on: [:x_forwarded_host, :x_forwarded_port, :x_forwarded_proto]For further details refer to Plug.RewriteOn.Plug Options:rewrite_on - rewrites the given connection information based on the given headers:hsts - a boolean on enabling HSTS or not, defaults to true:expires - seconds to expires for HSTS, defaults to 31_536_000 (1 year):preload - a boolean to request inclusion on the HSTS preload list (for full set of required flags, see: Chromium HSTS submission site), defaults to false:subdomains - a boolean on including subdomains or not in HSTS, defaults to false:exclude - exclude the given hosts from redirecting to the https scheme. Defaults to [&quot;localhost&quot;]. It may be set to a list of binaries or a tuple {module, function, args}.:host - a new host to redirect to if the request's scheme is http, defaults to conn.host. It may be set to a binary or a tuple {module, function, args} that will be invoked on demand:log - The log level at which this plug should log its request info. Default is :info. Can be false to disable logging.PortIt is not possible to directly configure the port in Plug.SSL because HSTS expects the port to be 443 for SSL. If you are not using HSTS and want to redirect to HTTPS on another port, you can sneak it alongside the host, for example: host: &quot;example.com:443&quot;.Excluded hosts tupleTuple {module, function, args} can be passed to be invoked each time the plug is checking whether to redirect host. Provided function needs to receive at least one argument (host).For example, you may define it as:plug Plug.SSL, rewrite_on: [:x_forwarded_proto], exclude: {__MODULE__, :excluded_host?, []}where:def excluded_host?(host) do # Custom logic end","ref":"Plug.SSL.html","title":"Plug.SSL","type":"module"},{"doc":"Configures and validates the options given to the :ssl application.This function is often called internally by adapters, such as Cowboy, to validate and set reasonable defaults for SSL handling. Therefore Plug users are not expected to invoke it directly, rather you pass the relevant SSL options to your adapter which then invokes this.OptionsThis function accepts all options defined in Erlang/OTP :ssl documentation.Besides the options from :ssl, this function adds on extra option::cipher_suite - it may be :strong or :compatible, as outlined in the following sectionFurthermore, it sets the following defaults:secure_renegotiate: true - to avoid certain types of man-in-the-middle attacksreuse_sessions: true - for improved handshake performance of recurring connectionsFor a complete guide on HTTPS and best pratices, see our Plug HTTPS Guide.Cipher SuitesTo simplify configuration of TLS defaults, this function provides two preconfigured options: cipher_suite: :strong and cipher_suite: :compatible. The Ciphers chosen and related configuration come from the OWASP Cipher String Cheat SheetWe've made two modifications to the suggested config from the OWASP recommendations. First we include ECDSA certificates which are excluded from their configuration. Second we have changed the order of the ciphers to deprioritize DHE because of performance implications noted within the OWASP post itself. As the article notes &quot;...the TLS handshake with DHE hinders the CPU about 2.4 times more than ECDHE&quot;.The Strong cipher suite only supports tlsv1.2. Ciphers were based on the OWASP Group A+ and includes support for RSA or ECDSA certificates. The intention of this configuration is to provide as secure as possible defaults knowing that it will not be fully compatible with older browsers and operating systems.The Compatible cipher suite supports tlsv1, tlsv1.1 and tlsv1.2. Ciphers were based on the OWASP Group B and includes support for RSA or ECDSA certificates. The intention of this configuration is to provide as secure as possible defaults that still maintain support for older browsers and Android versions 4.3 and earlierFor both suites we've specified certificate curves secp256r1, ecp384r1 and secp521r1. Since OWASP doesn't prescribe curves we've based the selection on Mozilla's recommendationsThe cipher suites were last updated on 2018-JUN-14.","ref":"Plug.SSL.html#configure/1","title":"Plug.SSL.configure/1","type":"function"},{"doc":"A plug to handle session cookies and session stores.The session is accessed via functions on Plug.Conn. Cookies and session have to be fetched with Plug.Conn.fetch_session/1 before the session can be accessed.The session is also lazy. Once configured, a cookie header with the session will only be sent to the client if something is written to the session in the first place.When using Plug.Session, also consider using Plug.CSRFProtection to avoid Cross Site Request Forgery attacks.Session storesSee Plug.Session.Store for the specification session stores are required to implement.Plug ships with the following session stores:Plug.Session.ETSPlug.Session.COOKIEOptions:store - session store module (required);:key - session cookie key (required);:domain - see Plug.Conn.put_resp_cookie/4;:max_age - see Plug.Conn.put_resp_cookie/4;:path - see Plug.Conn.put_resp_cookie/4;:secure - see Plug.Conn.put_resp_cookie/4;:http_only - see Plug.Conn.put_resp_cookie/4;:same_site - see Plug.Conn.put_resp_cookie/4;:extra - see Plug.Conn.put_resp_cookie/4;Additional options can be given to the session store, see the store's documentation for the options it accepts.Examplesplug Plug.Session, store: :ets, key: &quot;_my_app_session&quot;, table: :session","ref":"Plug.Session.html","title":"Plug.Session","type":"module"},{"doc":"A plug for serving static assets.It requires two options::at - the request path to reach for static assets. It must be a string.:from - the file system path to read static assets from. It can be either: a string containing a file system path, an atom representing the application name (where assets will be served from priv/static), or a tuple containing the application name and the directory to serve assets from (besides priv/static).The preferred form is to use :from with an atom or tuple, since it will make your application independent from the starting directory. For example, if you pass:plug Plug.Static, from: &quot;priv/app/path&quot;Plug.Static will be unable to serve assets if you build releases or if you change the current directory. Instead do:plug Plug.Static, from: {:app_name, &quot;priv/app/path&quot;}If a static asset cannot be found, Plug.Static simply forwards the connection to the rest of the pipeline.Cache mechanismsPlug.Static uses etags for HTTP caching. This means browsers/clients should cache assets on the first request and validate the cache on following requests, not downloading the static asset once again if it has not changed. The cache-control for etags is specified by the cache_control_for_etags option and defaults to &quot;public&quot;.However, Plug.Static also supports direct cache control by using versioned query strings. If the request query string starts with &quot;?vsn=&quot;, Plug.Static assumes the application is versioning assets and does not set the ETag header, meaning the cache behaviour will be specified solely by the cache_control_for_vsn_requests config, which defaults to &quot;public, max-age=31536000&quot;.Options:gzip - given a request for FILE, serves FILE.gz if it exists in the static directory and if the accept-encoding header is set to allow gzipped content (defaults to false).:brotli - given a request for FILE, serves FILE.br if it exists in the static directory and if the accept-encoding header is set to allow brotli-compressed content (defaults to false). FILE.br is checked first and dominates FILE.gz due to the better compression ratio.:cache_control_for_etags - sets the cache header for requests that use etags. Defaults to &quot;public&quot;.:etag_generation - specify a {module, function, args} to be used to generate an etag. The path of the resource will be passed to the function, as well as the args. If this option is not supplied, etags will be generated based off of file size and modification time. Note it is recommended for the etag value to be quoted, which Plug won't do automatically.:cache_control_for_vsn_requests - sets the cache header for requests starting with &quot;?vsn=&quot; in the query string. Defaults to &quot;public, max-age=31536000&quot;.:only - filters which requests to serve. This is useful to avoid file system access on every request when this plug is mounted at &quot;/&quot;. For example, if only: [&quot;images&quot;, &quot;favicon.ico&quot;] is specified, only files in the &quot;images&quot; directory and the &quot;favicon.ico&quot; file will be served by Plug.Static. Note that Plug.Static matches these filters against request uri and not against the filesystem. When requesting a file with name containing non-ascii or special characters, you should use urlencoded form. For example, you should write only: [&quot;file%20name&quot;] instead of only: [&quot;file name&quot;]. Defaults to nil (no filtering).:only_matching - a relaxed version of :only that will serve any request as long as one of the given values matches the given path. For example, only_matching: [&quot;images&quot;, &quot;favicon&quot;] will match any request that starts at &quot;images&quot; or &quot;favicon&quot;, be it &quot;/images/foo.png&quot;, &quot;/images-high/foo.png&quot;, &quot;/favicon.ico&quot; or &quot;/favicon-high.ico&quot;. Such matches are useful when serving digested files at the root. Defaults to nil (no filtering).:headers - other headers to be set when serving static assets. Specify either an enum of key-value pairs or a {module, function, args} to return an enum. The conn will be passed to the function, as well as the args.:content_types - custom MIME type mapping. As a map with filename as key and content type as value. For example: content_types: %{&quot;apple-app-site-association&quot; =&gt; &quot;application/json&quot;}.ExamplesThis plug can be mounted in a Plug.Builder pipeline as follows:defmodule MyPlug do use Plug.Builder plug Plug.Static, at: &quot;/public&quot;, from: :my_app, only: ~w(images robots.txt) plug :not_found def not_found(conn, _) do send_resp(conn, 404, &quot;not found&quot;) end end","ref":"Plug.Static.html","title":"Plug.Static","type":"module"},{"doc":"A plug to instrument the pipeline with :telemetry events.When plugged, the event prefix is a required option:plug Plug.Telemetry, event_prefix: [:my, :plug]In the example above, two events will be emitted:[:my, :plug, :start] - emitted when the plug is invoked. The event carries the system_time as measurement. The metadata is the whole Plug.Conn under the :conn key and any leftover options given to the plug under :options.[:my, :plug, :stop] - emitted right before the request is sent. The event carries a single measurement, :duration, which is the monotonic time difference between the stop and start events. It has the same metadata as the start event, except the connection has been updated.Note this plug measures the time between its invocation until a response is sent. The :stop event is not guaranteed to be emitted in all error cases, so this Plug cannot be used as a Telemetry span.Time unitThe :duration measurements are presented in the :native time unit. You can read more about it in the docs for System.convert_time_unit/3.Exampledefmodule InstrumentedPlug do use Plug.Router plug :match plug Plug.Telemetry, event_prefix: [:my, :plug] plug Plug.Parsers, parsers: [:urlencoded, :multipart] plug :dispatch get &quot;/&quot; do send_resp(conn, 200, &quot;Hello, world!&quot;) end endIn this example, the stop event's duration includes the time it takes to parse the request, dispatch it to the correct handler, and execute the handler. The events are not emitted for requests not matching any handlers, since the plug is placed after the match plug.","ref":"Plug.Telemetry.html","title":"Plug.Telemetry","type":"module"},{"doc":"A module (not a plug) for debugging in development.This module is commonly used within a Plug.Builder or a Plug.Router and it wraps the call/2 function.Notice Plug.Debugger does not catch errors, as errors should still propagate so that the Elixir process finishes with the proper reason. This module does not perform any logging either, as all logging is done by the web server handler.Note: If this module is used with Plug.ErrorHandler, only one of them will effectively handle errors. For this reason, it is recommended that Plug.Debugger is used before Plug.ErrorHandler and only in particular environments, like :dev.In case of an error, the rendered page drops the content-security-policy header before rendering the error to ensure that the error is displayed correctly.Examplesdefmodule MyApp do use Plug.Builder if Mix.env == :dev do use Plug.Debugger, otp_app: :my_app end plug :boom def boom(conn, _) do # Error raised here will be caught and displayed in a debug page # complete with a stacktrace and other helpful info. raise &quot;oops&quot; end endOptions:otp_app - the OTP application that is using Plug. This option is used to filter stacktraces that belong only to the given application.:style - custom styles (see below):banner - the optional MFA ({module, function, args}) which receives exception details and returns banner contents to appear at the top of the page. May be any string, including markup.Custom stylesYou may pass a :style option to customize the look of the HTML page.use Plug.Debugger, style: [primary: &quot;#c0392b&quot;, logo: &quot;data:image/png;base64,...&quot;]The following keys are available::primary - primary color:accent - accent color:logo - logo URI, or nil to disableThe :logo is preferred to be a base64-encoded data URI so not to make any external requests, though external URLs (eg, https://...) are supported.Custom BannersYou may pass an MFA ({module, function, args}) to be invoked when an error is rendered which provides a custom banner at the top of the debugger page. The function receives the following arguments, with the passed args concatenated at the end:[conn, status, kind, reason, stacktrace]For example, the following :banner option:use Plug.Debugger, banner: {MyModule, :debug_banner, []}would invoke the function:MyModule.debug_banner(conn, status, kind, reason, stacktrace)Links to the text editorIf a PLUG_EDITOR environment variable is set, Plug.Debugger will use it to generate links to your text editor. The variable should be set with __FILE__ and __LINE__ placeholders which will be correctly replaced. For example (with the TextMate editor):txmt://open/?url=file://__FILE__&amp;line=__LINE__Or, using Visual Studio Code:vscode://file/__FILE__:__LINE__","ref":"Plug.Debugger.html","title":"Plug.Debugger","type":"module"},{"doc":"A module to be used in your existing plugs in order to provide error handling.defmodule AppRouter do use Plug.Router use Plug.ErrorHandler plug :match plug :dispatch get &quot;/hello&quot; do send_resp(conn, 200, &quot;world&quot;) end def handle_errors(conn, %{kind: _kind, reason: _reason, stack: _stack}) do send_resp(conn, conn.status, &quot;Something went wrong&quot;) end endOnce this module is used, a callback named handle_errors/2 should be defined in your plug. This callback will receive the connection already updated with a proper status code for the given exception. The second argument is a map containing:the exception kind (:throw, :error or :exit),the reason (an exception for errors or a term for others)the stacktraceAfter the callback is invoked, the error is re-raised.It is advised to do as little work as possible when handling errors and avoid accessing data like parameters and session, as the parsing of those is what could have led the error to trigger in the first place.Also notice that these pages are going to be shown in production. If you are looking for error handling to help during development, consider using Plug.Debugger.Note: If this module is used with Plug.Debugger, it must be used after Plug.Debugger.","ref":"Plug.ErrorHandler.html","title":"Plug.ErrorHandler","type":"module"},{"doc":"A protocol that extends exceptions to be status-code aware.By default, it looks for an implementation of the protocol, otherwise checks if the exception has the :plug_status field or simply returns 500.","ref":"Plug.Exception.html","title":"Plug.Exception","type":"protocol"},{"doc":"Receives an exception and returns the possible actions that could be triggered for that error. Should return a list of actions in the following structure:%{ label: &quot;Text that will be displayed in the button&quot;, handler: {Module, :function, [args]} }Where:label a string/binary that names this actionhandler a MFArgs that will be executed when this action is triggeredIt will be rendered in the Plug.Debugger generated error page as buttons showing the label that upon pressing executes the MFArgs defined in the handler.Examplesdefimpl Plug.Exception, for: ActionableExample do def actions(_), do: [%{label: &quot;Print HI&quot;, handler: {IO, :puts, [&quot;Hi!&quot;]}}] end","ref":"Plug.Exception.html#actions/1","title":"Plug.Exception.actions/1","type":"function"},{"doc":"Receives an exception and returns its HTTP status code.","ref":"Plug.Exception.html#status/1","title":"Plug.Exception.status/1","type":"function"},{"doc":"","ref":"Plug.Exception.html#t:action/0","title":"Plug.Exception.action/0","type":"type"},{"doc":"","ref":"Plug.Exception.html#t:t/0","title":"Plug.Exception.t/0","type":"type"},{"doc":"Specification of the connection adapter API implemented by webservers.","ref":"Plug.Conn.Adapter.html","title":"Plug.Conn.Adapter","type":"behaviour"},{"doc":"Sends a chunk in the chunked response.If the request has method &quot;HEAD&quot;, the adapter should not send the response to the client.Webservers are advised to return :ok and not modify any further state for each chunk. However, the test implementation returns the actual body and payload so it can be used during testing.","ref":"Plug.Conn.Adapter.html#c:chunk/2","title":"Plug.Conn.Adapter.chunk/2","type":"callback"},{"doc":"Returns the HTTP protocol and its version.","ref":"Plug.Conn.Adapter.html#c:get_http_protocol/1","title":"Plug.Conn.Adapter.get_http_protocol/1","type":"callback"},{"doc":"Returns peer information such as the address, port and ssl cert.","ref":"Plug.Conn.Adapter.html#c:get_peer_data/1","title":"Plug.Conn.Adapter.get_peer_data/1","type":"callback"},{"doc":"Send an informational response to the client.If the adapter does not support inform, then {:error, :not_supported} should be returned.","ref":"Plug.Conn.Adapter.html#c:inform/3","title":"Plug.Conn.Adapter.inform/3","type":"callback"},{"doc":"Push a resource to the client.If the adapter does not support server push then {:error, :not_supported} should be returned.","ref":"Plug.Conn.Adapter.html#c:push/3","title":"Plug.Conn.Adapter.push/3","type":"callback"},{"doc":"Reads the request body.Read the docs in Plug.Conn.read_body/2 for the supported options and expected behaviour.","ref":"Plug.Conn.Adapter.html#c:read_req_body/2","title":"Plug.Conn.Adapter.read_req_body/2","type":"callback"},{"doc":"Sends the given status, headers as the beginning of a chunked response to the client.Webservers are advised to return nil as the sent_body, as the body can no longer be manipulated. However, the test implementation returns the actual body so it can be used during testing.","ref":"Plug.Conn.Adapter.html#c:send_chunked/3","title":"Plug.Conn.Adapter.send_chunked/3","type":"callback"},{"doc":"Sends the given status, headers and file as a response back to the client.If the request has method &quot;HEAD&quot;, the adapter should not send the response to the client.Webservers are advised to return nil as the sent_body, as the body can no longer be manipulated. However, the test implementation returns the actual body so it can be used during testing.","ref":"Plug.Conn.Adapter.html#c:send_file/6","title":"Plug.Conn.Adapter.send_file/6","type":"callback"},{"doc":"Sends the given status, headers and body as a response back to the client.If the request has method &quot;HEAD&quot;, the adapter should not send the response to the client.Webservers are advised to return nil as the sent_body, as the body can no longer be manipulated. However, the test implementation returns the actual body so it can be used during testing.","ref":"Plug.Conn.Adapter.html#c:send_resp/4","title":"Plug.Conn.Adapter.send_resp/4","type":"callback"},{"doc":"","ref":"Plug.Conn.Adapter.html#t:http_protocol/0","title":"Plug.Conn.Adapter.http_protocol/0","type":"type"},{"doc":"","ref":"Plug.Conn.Adapter.html#t:payload/0","title":"Plug.Conn.Adapter.payload/0","type":"type"},{"doc":"","ref":"Plug.Conn.Adapter.html#t:peer_data/0","title":"Plug.Conn.Adapter.peer_data/0","type":"type"},{"doc":"Conveniences for encoding and decoding cookies.","ref":"Plug.Conn.Cookies.html","title":"Plug.Conn.Cookies","type":"module"},{"doc":"Decodes the given cookies as given in either a request or response header.If a cookie is invalid, it is automatically discarded from the result.Examplesiex&gt; decode(&quot;key1=value1;key2=value2&quot;) %{&quot;key1&quot; =&gt; &quot;value1&quot;, &quot;key2&quot; =&gt; &quot;value2&quot;}","ref":"Plug.Conn.Cookies.html#decode/1","title":"Plug.Conn.Cookies.decode/1","type":"function"},{"doc":"Encodes the given cookies as expected in a response header.","ref":"Plug.Conn.Cookies.html#encode/2","title":"Plug.Conn.Cookies.encode/2","type":"function"},{"doc":"Conveniences for decoding and encoding url encoded queries.Plug allows a developer to build query strings that map to Elixir structures in order to make manipulation of such structures easier on the server side. Here are some examples:iex&gt; decode(&quot;foo=bar&quot;)[&quot;foo&quot;] &quot;bar&quot;If a value is given more than once, the last value takes precedence:iex&gt; decode(&quot;foo=bar&amp;foo=baz&quot;)[&quot;foo&quot;] &quot;baz&quot;Nested structures can be created via [key]:iex&gt; decode(&quot;foo[bar]=baz&quot;)[&quot;foo&quot;][&quot;bar&quot;] &quot;baz&quot;Lists are created with []:iex&gt; decode(&quot;foo[]=bar&amp;foo[]=baz&quot;)[&quot;foo&quot;] [&quot;bar&quot;, &quot;baz&quot;]Maps can be encoded:iex&gt; encode(%{foo: &quot;bar&quot;, baz: &quot;bat&quot;}) &quot;baz=bat&amp;foo=bar&quot;Encoding keyword lists preserves the order of the fields:iex&gt; encode([foo: &quot;bar&quot;, baz: &quot;bat&quot;]) &quot;foo=bar&amp;baz=bat&quot;When encoding keyword lists with duplicate keys, the key that comes first takes precedence:iex&gt; encode([foo: &quot;bar&quot;, foo: &quot;bat&quot;]) &quot;foo=bar&quot;Encoding named lists:iex&gt; encode(%{foo: [&quot;bar&quot;, &quot;baz&quot;]}) &quot;foo[]=bar&amp;foo[]=baz&quot;Encoding nested structures:iex&gt; encode(%{foo: %{bar: &quot;baz&quot;}}) &quot;foo[bar]=baz&quot;","ref":"Plug.Conn.Query.html","title":"Plug.Conn.Query","type":"module"},{"doc":"Decodes the given binary.The binary is assumed to be encoded in &quot;x-www-form-urlencoded&quot; format. The format is decoded and then validated for proper UTF-8 encoding.","ref":"Plug.Conn.Query.html#decode/4","title":"Plug.Conn.Query.decode/4","type":"function"},{"doc":"Decodes the given tuple and stores it in the accumulator.It parses the key and stores the value into the current accumulator. The keys and values are not assumed to be encoded in &quot;x-www-form-urlencoded&quot;.Parameter lists are added to the accumulator in reverse order, so be sure to pass the parameters in reverse order.","ref":"Plug.Conn.Query.html#decode_pair/2","title":"Plug.Conn.Query.decode_pair/2","type":"function"},{"doc":"Encodes the given map or list of tuples.","ref":"Plug.Conn.Query.html#encode/2","title":"Plug.Conn.Query.encode/2","type":"function"},{"doc":"Conveniences for working with status codes.","ref":"Plug.Conn.Status.html","title":"Plug.Conn.Status","type":"module"},{"doc":"Returns the status code given an integer or a known atom.Known status codesThe following status codes can be given as atoms with their respective value shown next::continue - 100:switching_protocols - 101:processing - 102:early_hints - 103:ok - 200:created - 201:accepted - 202:non_authoritative_information - 203:no_content - 204:reset_content - 205:partial_content - 206:multi_status - 207:already_reported - 208:im_used - 226:multiple_choices - 300:moved_permanently - 301:found - 302:see_other - 303:not_modified - 304:use_proxy - 305:switch_proxy - 306:temporary_redirect - 307:permanent_redirect - 308:bad_request - 400:unauthorized - 401:payment_required - 402:forbidden - 403:not_found - 404:method_not_allowed - 405:not_acceptable - 406:proxy_authentication_required - 407:request_timeout - 408:conflict - 409:gone - 410:length_required - 411:precondition_failed - 412:request_entity_too_large - 413:request_uri_too_long - 414:unsupported_media_type - 415:requested_range_not_satisfiable - 416:expectation_failed - 417:im_a_teapot - 418:misdirected_request - 421:unprocessable_entity - 422:locked - 423:failed_dependency - 424:too_early - 425:upgrade_required - 426:precondition_required - 428:too_many_requests - 429:request_header_fields_too_large - 431:unavailable_for_legal_reasons - 451:internal_server_error - 500:not_implemented - 501:bad_gateway - 502:service_unavailable - 503:gateway_timeout - 504:http_version_not_supported - 505:variant_also_negotiates - 506:insufficient_storage - 507:loop_detected - 508:not_extended - 510:network_authentication_required - 511","ref":"Plug.Conn.Status.html#code/1","title":"Plug.Conn.Status.code/1","type":"function"},{"doc":"Returns the atom for given integer.See code/1 for the mapping.","ref":"Plug.Conn.Status.html#reason_atom/1","title":"Plug.Conn.Status.reason_atom/1","type":"function"},{"doc":"","ref":"Plug.Conn.Status.html#reason_phrase/1","title":"Plug.Conn.Status.reason_phrase/1","type":"function"},{"doc":"A struct used as default on unfetched fields.The :aspect key of the struct specifies what field is still unfetched.Examplesunfetched = %Plug.Conn.Unfetched{aspect: :cookies}","ref":"Plug.Conn.Unfetched.html","title":"Plug.Conn.Unfetched","type":"module"},{"doc":"Callback implementation for Access.fetch/2.","ref":"Plug.Conn.Unfetched.html#fetch/2","title":"Plug.Conn.Unfetched.fetch/2","type":"function"},{"doc":"","ref":"Plug.Conn.Unfetched.html#get/3","title":"Plug.Conn.Unfetched.get/3","type":"function"},{"doc":"Callback implementation for Access.get_and_update/3.","ref":"Plug.Conn.Unfetched.html#get_and_update/3","title":"Plug.Conn.Unfetched.get_and_update/3","type":"function"},{"doc":"Callback implementation for Access.pop/2.","ref":"Plug.Conn.Unfetched.html#pop/2","title":"Plug.Conn.Unfetched.pop/2","type":"function"},{"doc":"","ref":"Plug.Conn.Unfetched.html#t:t/0","title":"Plug.Conn.Unfetched.t/0","type":"type"},{"doc":"Utilities for working with connection data","ref":"Plug.Conn.Utils.html","title":"Plug.Conn.Utils","type":"module"},{"doc":"Parses content type (without wildcards).It is similar to media_type/1 except wildcards are not accepted in the type nor in the subtype.Examplesiex&gt; content_type &quot;x-sample/json; charset=utf-8&quot; {:ok, &quot;x-sample&quot;, &quot;json&quot;, %{&quot;charset&quot; =&gt; &quot;utf-8&quot;}} iex&gt; content_type &quot;x-sample/json ; charset=utf-8 ; foo=bar&quot; {:ok, &quot;x-sample&quot;, &quot;json&quot;, %{&quot;charset&quot; =&gt; &quot;utf-8&quot;, &quot;foo&quot; =&gt; &quot;bar&quot;}} iex&gt; content_type &quot;\\r\\n text/plain;\\r\\n charset=utf-8\\r\\n&quot; {:ok, &quot;text&quot;, &quot;plain&quot;, %{&quot;charset&quot; =&gt; &quot;utf-8&quot;}} iex&gt; content_type &quot;text/plain&quot; {:ok, &quot;text&quot;, &quot;plain&quot;, %{}} iex&gt; content_type &quot;x/*&quot; :error iex&gt; content_type &quot;*/*&quot; :error","ref":"Plug.Conn.Utils.html#content_type/1","title":"Plug.Conn.Utils.content_type/1","type":"function"},{"doc":"Parses a comma-separated list of header values.Examplesiex&gt; list(&quot;foo, bar&quot;) [&quot;foo&quot;, &quot;bar&quot;] iex&gt; list(&quot;foobar&quot;) [&quot;foobar&quot;] iex&gt; list(&quot;&quot;) [] iex&gt; list(&quot;empties, , are,, filtered&quot;) [&quot;empties&quot;, &quot;are&quot;, &quot;filtered&quot;]","ref":"Plug.Conn.Utils.html#list/1","title":"Plug.Conn.Utils.list/1","type":"function"},{"doc":"Parses media types (with wildcards).Type and subtype are case insensitive while the sensitiveness of params depends on their keys and therefore are not handled by this parser.Returns:{:ok, type, subtype, map_of_params} if everything goes fine:error if the media type isn't validExamplesiex&gt; media_type &quot;text/plain&quot; {:ok, &quot;text&quot;, &quot;plain&quot;, %{}} iex&gt; media_type &quot;APPLICATION/vnd.ms-data+XML&quot; {:ok, &quot;application&quot;, &quot;vnd.ms-data+xml&quot;, %{}} iex&gt; media_type &quot;text/*; q=1.0&quot; {:ok, &quot;text&quot;, &quot;*&quot;, %{&quot;q&quot; =&gt; &quot;1.0&quot;}} iex&gt; media_type &quot;*/*; q=1.0&quot; {:ok, &quot;*&quot;, &quot;*&quot;, %{&quot;q&quot; =&gt; &quot;1.0&quot;}} iex&gt; media_type &quot;x y&quot; :error iex&gt; media_type &quot;*/html&quot; :error iex&gt; media_type &quot;/&quot; :error iex&gt; media_type &quot;x/y z&quot; :error","ref":"Plug.Conn.Utils.html#media_type/1","title":"Plug.Conn.Utils.media_type/1","type":"function"},{"doc":"Parses headers parameters.Keys are case insensitive and downcased, invalid key-value pairs are discarded.Examplesiex&gt; params(&quot;foo=bar&quot;) %{&quot;foo&quot; =&gt; &quot;bar&quot;} iex&gt; params(&quot; foo=bar &quot;) %{&quot;foo&quot; =&gt; &quot;bar&quot;} iex&gt; params(&quot;FOO=bar&quot;) %{&quot;foo&quot; =&gt; &quot;bar&quot;} iex&gt; params(&quot;Foo=bar; baz=BOING&quot;) %{&quot;foo&quot; =&gt; &quot;bar&quot;, &quot;baz&quot; =&gt; &quot;BOING&quot;} iex&gt; params(&quot;foo=BAR ; wat&quot;) %{&quot;foo&quot; =&gt; &quot;BAR&quot;} iex&gt; params(&quot;foo=\\&quot;bar\\&quot;; baz=\\&quot;boing\\&quot;&quot;) %{&quot;foo&quot; =&gt; &quot;bar&quot;, &quot;baz&quot; =&gt; &quot;boing&quot;} iex&gt; params(&quot;foo=\\&quot;bar;\\&quot;; baz=\\&quot;boing\\&quot;&quot;) %{&quot;foo&quot; =&gt; &quot;bar;&quot;, &quot;baz&quot; =&gt; &quot;boing&quot;} iex&gt; params(&quot;=&quot;) %{}","ref":"Plug.Conn.Utils.html#params/1","title":"Plug.Conn.Utils.params/1","type":"function"},{"doc":"Parses a value as defined in RFC-1341.For convenience, trims whitespace at the end of the token. Returns false if the token is invalid.Examplesiex&gt; token(&quot;foo&quot;) &quot;foo&quot; iex&gt; token(&quot;foo-bar&quot;) &quot;foo-bar&quot; iex&gt; token(&quot;&lt;foo&gt;&quot;) false iex&gt; token(~s[&quot;&lt;foo&gt;&quot;]) &quot;&lt;foo&gt;&quot; iex&gt; token(~S[&quot;&lt;f\\oo&gt;\\&quot;&lt;b\\ar&gt;&quot;]) &quot;&lt;foo&gt;\\&quot;&lt;bar&gt;&quot; iex&gt; token(&quot;foo &quot;) &quot;foo&quot; iex&gt; token(&quot;foo bar&quot;) false","ref":"Plug.Conn.Utils.html#token/1","title":"Plug.Conn.Utils.token/1","type":"function"},{"doc":"Validates the given binary is valid UTF-8.","ref":"Plug.Conn.Utils.html#validate_utf8!/3","title":"Plug.Conn.Utils.validate_utf8!/3","type":"function"},{"doc":"","ref":"Plug.Conn.Utils.html#t:params/0","title":"Plug.Conn.Utils.params/0","type":"type"},{"doc":"Parses JSON request body.JSON documents that aren't maps (arrays, strings, numbers, etc) are parsed into a &quot;_json&quot; key to allow proper param merging.An empty request body is parsed as an empty map.OptionsAll options supported by Plug.Conn.read_body/2 are also supported here. They are repeated here for convenience::length - sets the maximum number of bytes to read from the request, defaults to 8_000_000 bytes:read_length - sets the amount of bytes to read at one time from the underlying socket to fill the chunk, defaults to 1_000_000 bytes:read_timeout - sets the timeout for each socket read, defaults to 15_000msSo by default, Plug.Parsers will read 1_000_000 bytes at a time from the socket with an overall limit of 8_000_000 bytes.","ref":"Plug.Parsers.JSON.html","title":"Plug.Parsers.JSON","type":"module"},{"doc":"Parses multipart request body.OptionsAll options supported by Plug.Conn.read_body/2 are also supported here. They are repeated here for convenience::length - sets the maximum number of bytes to read from the request, defaults to 8_000_000 bytes:read_length - sets the amount of bytes to read at one time from the underlying socket to fill the chunk, defaults to 1_000_000 bytes:read_timeout - sets the timeout for each socket read, defaults to 15_000msSo by default, Plug.Parsers will read 1_000_000 bytes at a time from the socket with an overall limit of 8_000_000 bytes.Besides the options supported by Plug.Conn.read_body/2, the multipart parser also checks for::headers - containing the same :length, :read_length and :read_timeout options which are used explicitly for parsing multipart headers:include_unnamed_parts_at - string specifying a body parameter that can hold a lists of body parts that didn't have a 'Content-Disposition' header. For instance, include_unnamed_parts_at: &quot;_parts&quot; would result in a body parameter &quot;_parts&quot;, containing a list of parts, each with :body and :headers fields, like [%{body: &quot;{}&quot;, headers: [{&quot;content-type&quot;, &quot;application/json&quot;}]}]:validate_utf8 - specifies whether multipart body parts should be validated as utf8 binaries. Defaults to trueDynamic configurationIf you need to dynamically configure how Plug.Parsers.MULTIPART behave, for example, based on the connection or another system parameter, one option is to create your own parser that wraps it:defmodule MyMultipart do @multipart Plug.Parsers.MULTIPART def init(opts) do opts end def parse(conn, &quot;multipart&quot;, subtype, headers, opts) do limit = [limit: System.fetch_env!(&quot;UPLOAD_LIMIT&quot;)] opts = @multipart.init([limit: limit] ++ opts) @multipart.parse(conn, &quot;multipart&quot;, subtype, headers, opts) end def parse(conn, _type, _subtype, _headers, _opts) do {:next, conn} end end","ref":"Plug.Parsers.MULTIPART.html","title":"Plug.Parsers.MULTIPART","type":"module"},{"doc":"Parses urlencoded request body.OptionsAll options supported by Plug.Conn.read_body/2 are also supported here. They are repeated here for convenience::length - sets the maximum number of bytes to read from the request, defaults to 1_000_000 bytes:read_length - sets the amount of bytes to read at one time from the underlying socket to fill the chunk, defaults to 1_000_000 bytes:read_timeout - sets the timeout for each socket read, defaults to 15_000msSo by default, Plug.Parsers will read 1_000_000 bytes at a time from the socket with an overall limit of 8_000_000 bytes.","ref":"Plug.Parsers.URLENCODED.html","title":"Plug.Parsers.URLENCODED","type":"module"},{"doc":"Stores the session in a cookie.This cookie store is based on Plug.Crypto.MessageVerifier and Plug.Crypto.MessageEncryptor which encrypts and signs each cookie to ensure they can't be read nor tampered with.Since this store uses crypto features, it requires you to set the :secret_key_base field in your connection. This can be easily achieved with a plug:plug :put_secret_key_base def put_secret_key_base(conn, _) do put_in conn.secret_key_base, &quot;-- LONG STRING WITH AT LEAST 64 BYTES --&quot; endOptions:secret_key_base - the secret key base to built the cookie signing/encryption on top of. If one is given on initialization, the cookie store can precompute all relevant values at compilation time. Otherwise, the value is taken from conn.secret_key_base and cached.:encryption_salt - a salt used with conn.secret_key_base to generate a key for encrypting/decrypting a cookie, can be either a binary or an MFA returning a binary;:signing_salt - a salt used with conn.secret_key_base to generate a key for signing/verifying a cookie, can be either a binary or an MFA returning a binary;:key_iterations - option passed to Plug.Crypto.KeyGenerator when generating the encryption and signing keys. Defaults to 1000;:key_length - option passed to Plug.Crypto.KeyGenerator when generating the encryption and signing keys. Defaults to 32;:key_digest - option passed to Plug.Crypto.KeyGenerator when generating the encryption and signing keys. Defaults to :sha256;:serializer - cookie serializer module that defines encode/1 and decode/1 returning an {:ok, value} tuple. Defaults to :external_term_format.:log - Log level to use when the cookie cannot be decoded. Defaults to :debug, can be set to false to disable it.Examples# Use the session plug with the table name plug Plug.Session, store: :cookie, key: &quot;_my_app_session&quot;, encryption_salt: &quot;cookie store encryption salt&quot;, signing_salt: &quot;cookie store signing salt&quot;, key_length: 64, log: :debug","ref":"Plug.Session.COOKIE.html","title":"Plug.Session.COOKIE","type":"module"},{"doc":"Stores the session in an in-memory ETS table.This store does not create the ETS table; it expects that an existing named table with public properties is passed as an argument.We don't recommend using this store in production as every session will be stored in ETS and never cleaned until you create a task responsible for cleaning up old entries.Also, since the store is in-memory, it means sessions are not shared between servers. If you deploy to more than one machine, using this store is again not recommended.This store, however, can be used as an example for creating custom storages, based on Redis, Memcached, or a database itself.Options:table - ETS table name (required)For more information on ETS tables, visit the Erlang documentation at http://www.erlang.org/doc/man/ets.html.StorageThe data is stored in ETS in the following format:{sid :: String.t, data :: map, timestamp :: :erlang.timestamp}The timestamp is updated whenever there is a read or write to the table and it may be used to detect if a session is still active.Examples# Create an ETS table when the application starts :ets.new(:session, [:named_table, :public, read_concurrency: true]) # Use the session plug with the table name plug Plug.Session, store: :ets, key: &quot;sid&quot;, table: :session","ref":"Plug.Session.ETS.html","title":"Plug.Session.ETS","type":"module"},{"doc":"Specification for session stores.","ref":"Plug.Session.Store.html","title":"Plug.Session.Store","type":"behaviour"},{"doc":"Removes the session associated with given session id from the store.","ref":"Plug.Session.Store.html#c:delete/3","title":"Plug.Session.Store.delete/3","type":"callback"},{"doc":"Gets the store name from an atom or a module.iex&gt; Plug.Session.Store.get(CustomStore) CustomStore iex&gt; Plug.Session.Store.get(:cookie) Plug.Session.COOKIE","ref":"Plug.Session.Store.html#get/1","title":"Plug.Session.Store.get/1","type":"function"},{"doc":"Parses the given cookie.Returns a session id and the session contents. The session id is any value that can be used to identify the session by the store.The session id may be nil in case the cookie does not identify any value in the store. The session contents must be a map.","ref":"Plug.Session.Store.html#c:get/3","title":"Plug.Session.Store.get/3","type":"callback"},{"doc":"Initializes the store.The options returned from this function will be given to get/3, put/4 and delete/3.","ref":"Plug.Session.Store.html#c:init/1","title":"Plug.Session.Store.init/1","type":"callback"},{"doc":"Stores the session associated with given session id.If nil is given as id, a new session id should be generated and returned.","ref":"Plug.Session.Store.html#c:put/4","title":"Plug.Session.Store.put/4","type":"callback"},{"doc":"The cookie value that will be sent in cookie headers. This value should be base64 encoded to avoid security issues.","ref":"Plug.Session.Store.html#t:cookie/0","title":"Plug.Session.Store.cookie/0","type":"type"},{"doc":"The session contents, the final data to be stored after it has been built with Plug.Conn.put_session/3 and the other session manipulating functions.","ref":"Plug.Session.Store.html#t:session/0","title":"Plug.Session.Store.session/0","type":"type"},{"doc":"The internal reference to the session in the store.","ref":"Plug.Session.Store.html#t:sid/0","title":"Plug.Session.Store.sid/0","type":"type"},{"doc":"The request will not be processed due to a client error.","ref":"Plug.BadRequestError.html","title":"Plug.BadRequestError","type":"exception"},{"doc":"Error raised when CSRF token is invalid.","ref":"Plug.CSRFProtection.InvalidCSRFTokenError.html","title":"Plug.CSRFProtection.InvalidCSRFTokenError","type":"exception"},{"doc":"Error raised when non-XHR requests are used for Javascript responses.","ref":"Plug.CSRFProtection.InvalidCrossOriginRequestError.html","title":"Plug.CSRFProtection.InvalidCrossOriginRequestError","type":"exception"},{"doc":"Error raised when trying to modify or send an already sent response","ref":"Plug.Conn.AlreadySentError.html","title":"Plug.Conn.AlreadySentError","type":"exception"},{"doc":"Error raised when the cookie exceeds the maximum size of 4096 bytes.","ref":"Plug.Conn.CookieOverflowError.html","title":"Plug.Conn.CookieOverflowError","type":"exception"},{"doc":"Error raised when trying to send a header that has errors, for example:the header key contains uppercase charsthe header value contains newlines \\n","ref":"Plug.Conn.InvalidHeaderError.html","title":"Plug.Conn.InvalidHeaderError","type":"exception"},{"doc":"Raised when the request string is malformed, for example:the query has bad utf-8 encodingthe query fails to www-form decode","ref":"Plug.Conn.InvalidQueryError.html","title":"Plug.Conn.InvalidQueryError","type":"exception"},{"doc":"Error raised when no response is sent in a request","ref":"Plug.Conn.NotSentError.html","title":"Plug.Conn.NotSentError","type":"exception"},{"doc":"Wraps the connection in an error which is meant to be handled upper in the stack.Used by both Plug.Debugger and Plug.ErrorHandler.","ref":"Plug.Conn.WrapperError.html","title":"Plug.Conn.WrapperError","type":"exception"},{"doc":"Callback implementation for Exception.message/1.","ref":"Plug.Conn.WrapperError.html#message/1","title":"Plug.Conn.WrapperError.message/1","type":"function"},{"doc":"Reraises an error or a wrapped one.","ref":"Plug.Conn.WrapperError.html#reraise/1","title":"Plug.Conn.WrapperError.reraise/1","type":"function"},{"doc":"","ref":"Plug.Conn.WrapperError.html#reraise/3","title":"Plug.Conn.WrapperError.reraise/3","type":"function"},{"doc":"","ref":"Plug.Conn.WrapperError.html#reraise/4","title":"Plug.Conn.WrapperError.reraise/4","type":"function"},{"doc":"Raised when the request body contains bad encoding.","ref":"Plug.Parsers.BadEncodingError.html","title":"Plug.Parsers.BadEncodingError","type":"exception"},{"doc":"Error raised when the request body is malformed.","ref":"Plug.Parsers.ParseError.html","title":"Plug.Parsers.ParseError","type":"exception"},{"doc":"Callback implementation for Exception.message/1.","ref":"Plug.Parsers.ParseError.html#message/1","title":"Plug.Parsers.ParseError.message/1","type":"function"},{"doc":"Error raised when the request is too large.","ref":"Plug.Parsers.RequestTooLargeError.html","title":"Plug.Parsers.RequestTooLargeError","type":"exception"},{"doc":"Error raised when the request body cannot be parsed.","ref":"Plug.Parsers.UnsupportedMediaTypeError.html","title":"Plug.Parsers.UnsupportedMediaTypeError","type":"exception"},{"doc":"Callback implementation for Exception.message/1.","ref":"Plug.Parsers.UnsupportedMediaTypeError.html#message/1","title":"Plug.Parsers.UnsupportedMediaTypeError.message/1","type":"function"},{"doc":"","ref":"Plug.Router.InvalidSpecError.html","title":"Plug.Router.InvalidSpecError","type":"exception"},{"doc":"","ref":"Plug.Router.MalformedURIError.html","title":"Plug.Router.MalformedURIError","type":"exception"},{"doc":"","ref":"Plug.Static.InvalidPathError.html","title":"Plug.Static.InvalidPathError","type":"exception"},{"doc":"Timeout while waiting for the request.","ref":"Plug.TimeoutError.html","title":"Plug.TimeoutError","type":"exception"},{"doc":"","ref":"Plug.UploadError.html","title":"Plug.UploadError","type":"exception"},{"doc":"Plug Plug is:A specification for composable modules between web applicationsConnection adapters for different web servers in the Erlang VMDocumentation for Plug is available online.","ref":"readme.html","title":"Plug","type":"extras"},{"doc":"In order to use Plug, you need a webserver and its bindings for Plug. The Cowboy webserver is the most common one, which can be installed by adding plug_cowboy as a dependency to your mix.exs:def deps do [ {:plug_cowboy, &quot;~&gt; 2.0&quot;} ] end","ref":"readme.html#installation","title":"Plug - Installation","type":"extras"},{"doc":"defmodule MyPlug do import Plug.Conn def init(options) do # initialize options options end def call(conn, _opts) do conn |&gt; put_resp_content_type(&quot;text/plain&quot;) |&gt; send_resp(200, &quot;Hello world&quot;) end endThe snippet above shows a very simple example on how to use Plug. Save that snippet to a file and run it inside the plug application with:$ iex -S mix iex&gt; c &quot;path/to/file.ex&quot; [MyPlug] iex&gt; {:ok, _} = Plug.Cowboy.http MyPlug, [] {:ok, #PID&lt;...&gt;}Access http://localhost:4000/ and we are done! For now, we have directly started the server in our terminal but, for production deployments, you likely want to start it in your supervision tree. See the Supervised handlers section next.","ref":"readme.html#hello-world","title":"Plug - Hello world","type":"extras"},{"doc":"On a production system, you likely want to start your Plug pipeline under your application's supervision tree. Start a new Elixir project with the --sup flag:$ mix new my_app --supand then update lib/my_app/application.ex as follows:defmodule MyApp do # See https://hexdocs.pm/elixir/Application.html # for more information on OTP Applications @moduledoc false use Application def start(_type, _args) do # List all child processes to be supervised children = [ {Plug.Cowboy, scheme: :http, plug: MyPlug, options: [port: 4001]} ] # See https://hexdocs.pm/elixir/Supervisor.html # for other strategies and supported options opts = [strategy: :one_for_one, name: MyApp.Supervisor] Supervisor.start_link(children, opts) end endNow run mix run --no-halt and it will start your application with a web server running at localhost:4001.","ref":"readme.html#supervised-handlers","title":"Plug - Supervised handlers","type":"extras"},{"doc":"BranchSupportv1.11Bug fixesv1.10Security patches onlyv1.9Security patches onlyv1.8Security patches onlyv1.7Security patches onlyv1.6Security patches onlyv1.5Unsupported from 03/2021v1.4Unsupported from 12/2018v1.3Unsupported from 12/2018v1.2Unsupported from 06/2018v1.1Unsupported from 01/2018v1.0Unsupported from 05/2017","ref":"readme.html#supported-versions","title":"Plug - Supported Versions","type":"extras"},{"doc":"In the hello world example, we defined our first plug. What is a plug after all?A plug takes two shapes. A function plug receives a connection and a set of options as arguments and returns the connection:def hello_world_plug(conn, _opts) do conn |&gt; put_resp_content_type(&quot;text/plain&quot;) |&gt; send_resp(200, &quot;Hello world&quot;) endA module plug implements an init/1 function to initialize the options and a call/2 function which receives the connection and initialized options and returns the connection:defmodule MyPlug do def init([]), do: false def call(conn, _opts), do: conn endAs per the specification above, a connection is represented by the Plug.Conn struct:%Plug.Conn{host: &quot;www.example.com&quot;, path_info: [&quot;bar&quot;, &quot;baz&quot;], ...}Data can be read directly from the connection and also pattern matched on. Manipulating the connection often happens with the use of the functions defined in the Plug.Conn module. In our example, both put_resp_content_type/2 and send_resp/3 are defined in Plug.Conn.Remember that, as everything else in Elixir, a connection is immutable, so every manipulation returns a new copy of the connection:conn = put_resp_content_type(conn, &quot;text/plain&quot;) conn = send_resp(conn, 200, &quot;ok&quot;) connFinally, keep in mind that a connection is a direct interface to the underlying web server. When you call send_resp/3 above, it will immediately send the given status and body back to the client. This makes features like streaming a breeze to work with.","ref":"readme.html#the-plug-conn-struct","title":"Plug - The Plug.Conn struct","type":"extras"},{"doc":"To write a &quot;router&quot; plug that dispatches based on the path and method of incoming requests, Plug provides Plug.Router:defmodule MyRouter do use Plug.Router plug :match plug :dispatch get &quot;/hello&quot; do send_resp(conn, 200, &quot;world&quot;) end forward &quot;/users&quot;, to: UsersRouter match _ do send_resp(conn, 404, &quot;oops&quot;) end endThe router is a plug. Not only that: it contains its own plug pipeline too. The example above says that when the router is invoked, it will invoke the :match plug, represented by a local (imported) match/2 function, and then call the :dispatch plug which will execute the matched code.Plug ships with many plugs that you can add to the router plug pipeline, allowing you to plug something before a route matches or before a route is dispatched to. For example, if you want to add logging to the router, just do:plug Plug.Logger plug :match plug :dispatchNote Plug.Router compiles all of your routes into a single function and relies on the Erlang VM to optimize the underlying routes into a tree lookup, instead of a linear lookup that would instead match route-per-route. This means route lookups are extremely fast in Plug!This also means that a catch all match block is recommended to be defined as in the example above, otherwise routing fails with a function clause error (as it would in any regular Elixir function).Each route needs to return the connection as per the Plug specification. See the Plug.Router docs for more information.","ref":"readme.html#plug-router","title":"Plug - Plug.Router","type":"extras"},{"doc":"Plug ships with a Plug.Test module that makes testing your plugs easy. Here is how we can test the router from above (or any other plug):defmodule MyPlugTest do use ExUnit.Case, async: true use Plug.Test @opts MyRouter.init([]) test &quot;returns hello world&quot; do # Create a test connection conn = conn(:get, &quot;/hello&quot;) # Invoke the plug conn = MyRouter.call(conn, @opts) # Assert the response and status assert conn.state == :sent assert conn.status == 200 assert conn.resp_body == &quot;world&quot; end end","ref":"readme.html#testing-plugs","title":"Plug - Testing plugs","type":"extras"},{"doc":"This project aims to ship with different plugs that can be re-used across applications:Plug.BasicAuth - provides Basic HTTP authentication;Plug.CSRFProtection - adds Cross-Site Request Forgery protection to your application. Typically required if you are using Plug.Session;Plug.Head - converts HEAD requests to GET requests;Plug.Logger - logs requests;Plug.MethodOverride - overrides a request method with one specified in the request parameters;Plug.Parsers - responsible for parsing the request body given its content-type;Plug.RequestId - sets up a request ID to be used in logs;Plug.RewriteOn - rewrite the request's host/port/protocol from x-forwarded-* headers;Plug.Session - handles session management and storage;Plug.SSL - enforces requests through SSL;Plug.Static - serves static files;Plug.Telemetry - instruments the plug pipeline with :telemetry events;You can go into more details about each of them in our docs.","ref":"readme.html#available-plugs","title":"Plug - Available plugs","type":"extras"},{"doc":"Modules that can be used after you use Plug.Router or Plug.Builder to help development:Plug.Debugger - shows a helpful debugging page every time there is a failure in a request;Plug.ErrorHandler - allows developers to customize error pages in case of crashes instead of sending a blank one;","ref":"readme.html#helper-modules","title":"Plug - Helper modules","type":"extras"},{"doc":"We welcome everyone to contribute to Plug and help us tackle existing issues!Use the issue tracker for bug reports or feature requests. Open a pull request when you are ready to contribute. When submitting a pull request you should not update the CHANGELOG.md.If you are planning to contribute documentation, please check our best practices for writing documentation.Finally, remember all interactions in our official spaces follow our Code of Conduct.","ref":"readme.html#contributing","title":"Plug - Contributing","type":"extras"},{"doc":"Plug source code is released under Apache License 2.0. Check LICENSE file for more information.","ref":"readme.html#license","title":"Plug - License","type":"extras"},{"doc":"HTTPSPlug can serve HTTP over TLS ('HTTPS') through an appropriately configured Adapter. While the exact syntax for defining an HTTPS listener is adapter-specific, Plug does define a common set of TLS configuration options that most adapters support, formally documented as Plug.SSL.configure/1.This guide describes how to use these parameters to set up an HTTPS server with Plug, and documents some best-practices and potential pitfalls.Editor's note: The secure transport protocol used by HTTPS is nowadays referred to as TLS. However, the application in the Erlang/OTP standard library that implements it is called :ssl, for historical reasons. In this document we will refer to the protocol as 'TLS' and to the Erlang/OTP implementation as :ssl, and its configuration parameters as :ssl options.","ref":"https.html","title":"HTTPS","type":"extras"},{"doc":"The prerequisites for running an HTTPS server with Plug include:The Erlang/OTP runtime, with OpenSSL bindings; run :crypto.info_lib() in an IEx session to verifyA Plug Adapter that supports HTTPS, e.g. Plug.CowboyA valid certificate and associated private keySelf-signed CertificateFor testing purposes it may be sufficient to use a self-signed certificate. Such certificates generally result in warnings in browsers and failed connections from other tools, but these can be overridden to enable HTTPS testing. This is especially useful for local testing of HTTP 2, which is only specified over TLS.Warning: use self-signed certificates only for local testing, and do not mark such test certificates as globally trusted in browsers or operating system!The Phoenix project includes a Mix task mix phx.gen.cert that generates the necessary files and places them in the application's 'priv' directory. The X509 package can be used as a dev-only dependency to add a similar mix x509.gen.selfsigned task to non-Phoenix projects.Alternatively, the OpenSSL CLI or other utilities can be used to generate a self-signed certificate. Instructions are widely available online.CA Issued CertificateFor staging and production it is necessary to obtain a CA-signed certificate from a trusted Certificate Authority, such as Let's Encrypt. Certificates issued by a CA usually come with an additional file containing one or more certificates that make up the 'CA chain'.For use with Plug the certificates and key should be stored in PEM format, containing Base64-encoded data between 'BEGIN' and 'END' markers. Some useful OpenSSL commands for converting certificates/keys from other formats can be found at the end of this document.","ref":"https.html#prerequisites","title":"HTTPS - Prerequisites","type":"extras"},{"doc":"A minimal HTTPS listener, using Plug.Cowboy, might be defined as follows:Plug.Cowboy.https MyApp.MyPlug, [], port: 8443, cipher_suite: :strong, certfile: &quot;/etc/letsencrypt/live/example.net/cert.pem&quot;, keyfile: &quot;/etc/letsencrypt/live/example.net/privkey.pem&quot;, cacertfile: &quot;/etc/letsencrypt/live/example.net/chain.pem&quot;The cacertfile option is not needed when using a self-signed certificate, or when the file pointed to by certfile contains both the server certificate and all necessary CA chain certificates: #... certfile: &quot;/etc/letsencrypt/live/example.net/fullchain.pem&quot;, keyfile: &quot;/etc/letsencrypt/live/example.net/privkey.pem&quot;It is possible to bundle the certificate files with the application, possibly for packaging into a release. In this case the files must be stored under the application's 'priv' directory. The otp_app option must be set to the name of the OTP application that contains the files, in order to correctly resolve the relative paths:Plug.Cowboy.https MyApp.MyPlug, [], port: 8443, cipher_suite: :strong, certfile: &quot;priv/cert/selfsigned.pem&quot;, keyfile: &quot;priv/cert/selfsigned_key.pem&quot;, otp_app: :my_appRemember to exclude the files from version control, unless the certificate and key are shared by all developers for testing purposes only. For example, add this line to the '.gitignore' file: priv/**/*.pem.","ref":"https.html#getting-started","title":"HTTPS - Getting Started","type":"extras"},{"doc":"In addition to a certificate, an HTTPS server needs a secure TLS protocol configuration. Plug.SSL always sets the following options:Set secure_renegotiate: true, to avoid certain types of man-in-the-middle attacksSet reuse_sessions: true, for improved handshake performance of recurring connectionsAdditional options can be set by selecting a predefined profile or by setting :ssl options individually.Predefined OptionsTo simplify configuration of TLS defaults Plug provides two preconfigured options: cipher_suite: :strong and cipher_suite: :compatible.The :strong profile enables AES-GCM ciphers with ECDHE or DHE key exchange, and TLS version 1.2 only. It is intended for typical installations with support for browsers and other modern clients.The :compatible profile additionally enables AES-CBC ciphers, as well as TLS versions 1.1 and 1.0. Use this configuration to allow connections from older clients, such as older PC or mobile operating systems. Note that RSA key exchange is not enabled by this configuration, due to known weaknesses, so to support clients that do not support ECDHE or DHE it is necessary specify the ciphers explicitly (see below).In addition, both profiles:Configure the server to choose a cipher based on its own preferences rather than the client's (honor_cipher_order set to true); when specifying a custom cipher list, ensure the ciphers are listed in descending order of preferenceSelect the 'Prime' (SECP) curves for use in Elliptic Curve Cryptography (ECC)All these parameters, including the global defaults mentioned above, can be overridden by specifying custom :ssl configuration options.It is worth noting that the cipher lists and TLS protocol versions selected by the profiles are whitelists. If a new Erlang/OTP release introduces new TLS protocol versions or ciphers that are not included in the profile definition, they would have to be enabled explicitly by overriding the :ciphers and/or :versions options, until such time as they are added to the Plug.SSL profiles.The ciphers chosen and related configuration are based on OWASP recommendations, with some modifications as described in the Plug.SSL.configure/1 documentation.Manual ConfigurationPlease refer to the Erlang/OTP :ssl documentation for details on the supported configuration options.An example configuration with custom :ssl options might look like this:Plug.Cowboy.https MyApp.MyPlug, [], port: 8443, certfile: &quot;/etc/letsencrypt/live/example.net/cert.pem&quot;, keyfile: &quot;/etc/letsencrypt/live/example.net/privkey.pem&quot;, cacertfile: &quot;/etc/letsencrypt/live/example.net/chain.pem&quot;, versions: [:&quot;tlsv1.2&quot;, :&quot;tlsv1.1&quot;], ciphers: [ &#39;ECDHE-RSA-AES256-GCM-SHA384&#39;, &#39;ECDHE-RSA-AES128-GCM-SHA256&#39;, &#39;DHE-RSA-AES256-GCM-SHA384&#39;, &#39;DHE-RSA-AES128-GCM-SHA256&#39; ], honor_cipher_order: true, sni_fun: &amp;MyPlug.ssl_opts_for_hostname/1","ref":"https.html#tls-protocol-options","title":"HTTPS - TLS Protocol Options","type":"extras"},{"doc":"Once a server is configured to support HTTPS it is often a good idea to redirect HTTP requests to HTTPS. To do this, include Plug.SSL in the Plug pipeline.To prevent downgrade attacks, in which an attacker intercepts a plain HTTP request to the server before the redirect to HTTPS takes place, Plug.SSL by default sets the 'Strict-Transport-Security' (HSTS) header. This informs the browser that the current site must only ever be accessed over HTTPS, even if the user typed or clicked a plain HTTP URL. This only works if the site is reachable on port 443 (see Listening on Port 443, below).Warning: it is very difficult, if not impossible, to revert the effect of HSTS before the entry stored in the browser expires! Consider using a short :expires value initially, and increasing it to a large value (e.g. 31536000 seconds for 1 year) after testing.The Strict-Transport-Security header can be disabled altogether by setting hsts: false in the Plug.SSL options.","ref":"https.html#http-strict-transport-security-hsts","title":"HTTPS - HTTP Strict Transport Security (HSTS)","type":"extras"},{"doc":"To protect the private key on disk it is best stored in encrypted PEM format, protected by a password. When configuring a Plug server with an encrypted private key, specify the password using the :password option:Plug.Cowboy.https MyApp.MyPlug, [], port: 8443, certfile: &quot;/etc/letsencrypt/live/example.net/cert.pem&quot;, keyfile: &quot;/etc/letsencrypt/live/example.net/privkey_aes.pem&quot;, cacertfile: &quot;/etc/letsencrypt/live/example.net/chain.pem&quot;, password: &quot;SECRET&quot;To encrypt an existing PEM-encoded RSA key use the OpenSSL CLI: openssl rsa -in privkey.pem -out privkey_aes.pem -aes128. Use ec instead of rsa when using an ECDSA certificate. Don't forget to securely erase the unencrypted copy afterwards! Best practice would be to encrypt the file immediately during initial key generation: please refer to the instructions provided by the CA.Note: at the time of writing, Erlang/OTP does not support keys encrypted with AES-256. The OpenSSL command in the previous paragraph can also be used to convert an AES-256 encrypted key to AES-128.","ref":"https.html#encrypted-keys","title":"HTTPS - Encrypted Keys","type":"extras"},{"doc":"Sometimes it is preferable to not store the private key on disk at all. Instead, the private key might be passed to the application using an environment variable or retrieved from a key store such as Vault.In such cases it is possible to pass the private key directly, using the :key parameter. For example, assuming an RSA private key is available in the PRIVKEY environment variable in Base64 encoded DER format, the key may be set as follows:der = System.get_env(&quot;PRIVKEY&quot;) |&gt; Base.decode64! Plug.Cowboy.https MyApp.MyPlug, [], port: 8443, key: {:RSAPrivateKey, der}, #....Note that reading environment variables in Mix config files only works when starting the application using Mix, e.g. in a development environment. In production, a different approach is needed for runtime configuration, but this is out of scope for the current document.The certificate and CA chain can also be specified using DER binaries, using the :cert and :cacerts options, but this is best avoided. The use of PEM files has been tested much more thoroughly with the Erlang/OTP :ssl application, and there have been a number of issues with DER binary certificates in the past.","ref":"https.html#passing-der-binaries","title":"HTTPS - Passing DER Binaries","type":"extras"},{"doc":"It is recommended to generate a custom set of Diffie-Hellman parameters, to be used for the DHE key exchange. Use the following OpenSSL CLI command to create a dhparam.pem file:openssl dhparam -out dhparams.pem 4096On a slow machine (e.g. a cheap VPS) this may take several hours. You may want to run the command on a strong machine and copy the file over to the target server: the file does not need to be kept secret. It is best practice to rotate the file periodically.Pass the (relative or absolute) path using the :dhfile option:Plug.Cowboy.https MyApp.MyPlug, [], port: 8443, cipher_suite: :strong, certfile: &quot;priv/cert/selfsigned.pem&quot;, keyfile: &quot;priv/cert/selfsigned_key.pem&quot;, dhfile: &quot;priv/cert/dhparams.pem&quot;, otp_app: :my_appIf no custom parameters are specified, Erlang's :ssl uses its built-in defaults. Since OTP 19 this has been the 2048-bit 'group 14' from RFC3526.","ref":"https.html#custom-diffie-hellman-parameters","title":"HTTPS - Custom Diffie-Hellman Parameters","type":"extras"},{"doc":"Whenever a certificate is about to expire, when the contents of the certificate have been updated, or when the certificate is 're-keyed', the HTTPS server needs to be updated with the new certificate and/or key.When using the :certfile and :keyfile parameters to reference PEM files on disk, replacing the certificate and key is as simple as overwriting the files. Erlang's :ssl application periodically reloads any referenced files, with changes taking effect in subsequent handshakes. It may be best to use symbolic links that point to versioned copies of the files, to allow for quick rollback in case of problems.Note that there is a potential race condition when both the certificate and the key need to be replaced at the same time: if the :ssl application reloads one file before the other file is updated, the partial update can leave the HTTPS server with a mismatched private key. This can be avoiding by placing the private key in the same PEM file as the certificate, and omitting the :keyfile option. This configuration allows atomic updates, and it works because :ssl looks for a private key entry in the :certfile PEM file if no :key or :keyfile option is specified.While it is possible to update the DER binaries passed in the :cert or :key options (as well as any other TLS protocol parameters) at runtime, this requires knowledge of the internals of the Plug adapter being used, and is therefore beyond the scope of this document.","ref":"https.html#renewing-certificates","title":"HTTPS - Renewing Certificates","type":"extras"},{"doc":"By default clients expect HTTPS servers to listen on port 443. It is possible to specify a different port in HTTPS URLs, but for public servers it is often preferable to stick to the default. In particular, HSTS requires that the site be reachable on port 443 using HTTPS.This presents a problem, however: only privileged processes can bind to TCP port numbers under 1024, and it is bad idea to run the application as 'root'.Leaving aside solutions that rely on external network elements, such as load balancers, there are several solutions on typical Linux servers:Deploy a reverse proxy or load balancer process, such as Nginx or HAProxy (see Offloading TLS, below); the proxy listens on port 443 and passes the traffic to the Elixir application running on an unprivileged portCreate an IPTables rule to forward packets arriving on port 443 to the port on which the Elixir application is runningGive the Erlang/OTP runtime (that is, the BEAM VM executable) permission to bind to privileged ports using 'setcap', e.g. sudo setcap 'cap_net_bind_service=+ep' /usr/lib/erlang/erts-10.1/bin/beam.smp; update the path as necessary, and remember to run the command again after Erlang upgradesUse a tool such as 'authbind' to give an unprivileged user/group permission to bind to specific portsThis is not intended to be an exhaustive list, as this topic is actually a bit beyond the scope of the current document. The issue is a generic one, not specific to Erlang/Elixir, and further explanations can be found online.","ref":"https.html#listening-on-port-443","title":"HTTPS - Listening on Port 443","type":"extras"},{"doc":"So far this document has focussed on configuring Plug to handle TLS within the application. Some people instead prefer to terminate TLS in a proxy or load balancer deployed in front of the Plug application.Pros and ConsOffloading might be done to achieve higher throughput, or to stick to the more widely used OpenSSL implementation of the TLS protocol. The Erlang/OTP implementation depends on OpenSSL for the underlying cryptography, but it implements its own message framing and protocol state machine. While it is not clear that one implementation is inherently more secure than the other, just patching OpenSSL along with everybody else in case of vulnerabilities might give peace of mind, compared to than having to research the implications on the Erlang/OTP implementation.On the other hand, the proxy solution might not support end-to-end HTTP 2, limiting the benefits of the new protocol. It can also introduce operational complexities and new resource constraints, especially for long-lived connections such as WebSockets.Plug Configuration ImpactWhen using TLS offloading it may be necessary to make some configuration changes to the application.Plug.SSL takes on another important role when using TLS offloading: it can update the :scheme and :port fields in the Plug.Conn struct based on an HTTP header (e.g. 'X-Forwarded-Proto'), to reflect the actual protocol used by the client (HTTP or HTTPS). It is very important that the :scheme field properly reflects the use of HTTPS, even if the connection between the proxy and the application uses plain HTTP, because cookies set by Plug.Session and Plug.Conn.put_resp_cookie/4 by default set the 'secure' cookie flag only if :scheme is set to :https! When relying on this default behaviour it is essential that Plug.SSL is included in the Plug pipeline, that its :rewrite_on option is set correctly, and that the proxy sets the appropriate header.The :remote_ip field in the Plug.Conn struct by default contains the network peer IP address. Terminating TLS in a separate process or network element typically masks the actual client IP address from the Elixir application. If proxying is done at the HTTP layer, the original client IP address is often inserted into an HTTP header, e.g. 'X-Forwarded-For'. There are Plug packages available to extract the client IP from such a header and update the :remote_ip field.Warning: ensure that clients cannot spoof their IP address by including this header in their original request, by filtering such headers in the proxy!For solutions that operate below the HTTP layer, e.g. using HAProxy, the client IP address can sometimes be passed through the 'PROXY protocol'. Extracting this information must be handled by the Plug adapter. Please refer to the Plug adapter documentation for further information.","ref":"https.html#offloading-tls","title":"HTTPS - Offloading TLS","type":"extras"},{"doc":"When certificate and/or key files are not in provided in PEM format they can usually be converted using the OpenSSL CLI. This section describes some common formats and the associated OpenSSL commands to convert to PEM.From DER to PEMDER-encoded files contain binary data. Common file extensions are .crt for certificates and .key for keys.To convert a single DER-encoded certificate to PEM format: openssl x509 -in server.crt -inform der -out cert.pemTo convert an RSA private key from DER to PEM format: openssl rsa -in privkey.der -inform der -out privkey.pem. If the private key is a Elliptic Curve key, for use with an ECDSA certificate, replace rsa with ec. You may want to add the -aes128 argument to produce an encrypted, password protected PEM file.From PKCS#12 to PEMThe PKCS#12 format is a container format containing one or more certificates and/or encrypted keys. Such files typically have a .p12 extension.To extract all certificates from a PKCS#12 file to a PEM file: openssl pkcs12 -in server.p12 -nokeys -out fullchain.pem. The resulting file contains all certificates from the input file, typically the server certificate and any CA certificates that make up the CA chain. You can split the file into seperate cert.pem and chain.pem files using a text editor, or you can just pass certfile: fullchain.pem to the HTTPS adapter.To extract a private key from a PKCS#12 file to a PEM file: openssl pkcs12 -in server.p12 -nocerts -nodes -out privkey.pem. You may want to replace the -nodes argument with -aes128 to produce an encrypted, password protected PEM file.","ref":"https.html#converting-certificates-and-keys","title":"HTTPS - Converting Certificates and Keys","type":"extras"}]